AnimationPlayer = Object:extend()

--[[
Why a difference between animations and animation players?
Animation players handle the drawing and updating of animations,
while animation objects are just stores of the frames, as well
as some other data. This allows multiple animation players with
different animation cycles to use the same animation without 
having to load the frames again.

Animation players should be tied to whatever object is using them
]]


function AnimationPlayer:new(animation, fps, frameOffset)
  if animation == nil then
    error("Animation set was nil!", 3)
  end

  
  self.fps = fps or animation.fps
  
  self.currentFrame = 1
  self.frameOffset = frameOffset or 0
  self.animation = animation
  self.timePlayed = nil
end


function AnimationPlayer:setAnimation(animation)
  self.animation = animation
  if not type(animation) == "table" then
    error("No value given! ("..type(animation)..")")
  end
end


function AnimationPlayer:update(dt)
  if self.timePlayed then
    self.timePlayed = self.timePlayed + dt
    self.currentFrame = math.floor((self.timePlayed*self.fps+self.frameOffset)%#self.animation.frames)+1
  else
    self.currentFrame = 1
  end
end



function AnimationPlayer:play(x, y, r, sx, sy)
  love.graphics.setBlendMode("alpha")
  self.timePlayed = self.timePlayed or 0

  blue.graphics.drawCentredShaded(self.animation.frames[self.currentFrame], x, y, r, sx, sy, self.animation.xOffset, self.animation.yOffset)
  
  return self.timePlayed*self.fps >= #self.animation.frames
end



function AnimationPlayer:stop()
  self.timePlayed = nil
end

