attributeEditor = {}
attributeEditor.buttons = {}

local width = 400
local scale = 1.5
local selectedObject = nil

local buttonSortTable = 
{
  "name",
  "logicType",
  "book",
  "arg1",
  "arg2",
  "arg3",
  "arg4",
  "red",
  "green",
  "blue",
  "intensity",
  "matchOutside",
  "textureName",
  "superTexture",
  "animation",
  "draw",
  "drawPriority", --I will not appologise
  "priorityDraw",
  "depth",
  "colour",
  "transparency",
  "wrapMode",
  "hiddenOutside",
  "fadeWhenBehind",
  "painted",
  "foreground",
  "isBackground",
  "minSpeed",
  "focusCamera",
  "zoomCamera",
  "singleUse",
  "doorLocked",
  "doorType",
  "openSpeed",
  "canClose",
  "autoCloseTimer",
  "priority",
  "maxTriggers",
  "litTriggerTarget",
  "triggerTarget",
  "triggerOnExtinguish",
  "interactRadius",
  "speedEnterRadius",
  "requiresFacing",
  "startsLit",
  "lightColour",
  "lightFlicker",
  "lightIntensity",
  "lightZ",
  "lightOffsetY",
  "lightOffsetX",
  "extinguishTimer",
  "rotation",
  "scaleX",
  "scaleY",
  "amount",
  "spreadX",
  "spreadY",
  "initialVX",
  "initialVY",
  "particleLife",
  "targetX",
  "targetY",
  "smooth",
  "collidable",
  "wetness",
  "hasPhysics",
  "hasGravity",
  "inverted",
  "springBack",
  "rotateBack",
  "stoneActive",
  "protected",
  "ignore"
}



buttonSortTable = swapKeyValues(buttonSortTable)


function attributeEditor:update(p)
  local i = 1
  selectedObject = p
  attributeEditor.buttons = {}
  
    
  
  if p.isTalkative then
    local i = 1
    
    
    while true do
      if (p.attributes["book"..i] and p.attributes["book"..i] ~= "") and p.attributes["book"..i+1] == nil then
        p.attributes["book"..i+1] = ""
        break
      end
      if p.attributes["book"..i] == "" then
        break
      end
      if not p.attributes.book2 then
        break
      end
      
      i = i + 1
    end
  end
  
  
  if p.attributes then
    for name, value in pairs(p.attributes) do
      attributeEditor.buttons[i] = {}
      attributeEditor.buttons[i].name = name
      attributeEditor.buttons[i].value = value
      attributeEditor.buttons[i].x = window.width - width + 10
      attributeEditor.buttons[i].width = 210+10
      attributeEditor.buttons[i].height = font:getHeight()*scale
      i = i + 1
    end
  end
  
  table.sort(attributeEditor.buttons, 
    function(k1, k2)
      
      
      local k1Name = k1.name or "other"
      local k2Name = k2.name or "other"

      if string.sub(k1Name, 1, 4) == "book" and string.sub(k2Name, 1, 4) == "book" then
        
        return string.sub(k1Name, 5, 5) < string.sub(k2Name, 5, 5)
        
      elseif string.sub(k1Name, 1, 4) == "book" then
        k1Name = "book"
        
      elseif string.sub(k2Name, 1, 4) == "book" then
        k2Name = "book"
      end
      
      if not buttonSortTable[k1Name] then
        error(k1Name.." does not exist in the attribute table!")
      elseif not buttonSortTable[k2Name] then
        error(k2Name.." does not exist in the attribute table!")
      end
  
      
      return buttonSortTable[k1Name] < buttonSortTable[k2Name]
    end
  )
  for i, button in ipairs(attributeEditor.buttons) do
    attributeEditor.buttons[i].y = i*(font:getHeight()+2)*scale+50
    attributeEditor.buttons[i].selected = pointInRect(attributeEditor.buttons[i], love.mouse.getPosition())
    local name = attributeEditor.buttons[i].name
    
    if attributeEditor.buttons[i].selected and clickedThisFrame.primary then 
      
      if p.attributes[name] == true or p.attributes[name] == false then
        p.attributes[name] = toggle(p.attributes[name], true, false)
        
        local positiveSound = sound.effect.ui.positive:clone()
        positiveSound:play()
        
      elseif type(p.attributes[name]) == "string" then
        
        local positiveSound = sound.effect.ui.positive:clone()
        positiveSound:play()
        
        if name == "name" then
          textEditor.callback = function()
            
            nameIndex:updateEntityName(p, textEditor.value)
            
            p.attributes[name] = textEditor.value
          end
        else
          textEditor.callback = function()
            p.attributes[name] = textEditor.value
          end
        end
        textEditor:show(p.attributes[name])
        elseif type(p.attributes[name]) == "number" then
          textEditor.callback = function()
          p.attributes[name] = tonumber(textEditor.value)
        end
        textEditor:show(p.attributes[name])
      else
        local negativeSound = sound.effect.ui.negative:clone()
        negativeSound:play()
      end
    end
    
  end
end


function attributeEditor:draw()
  love.graphics.setColor(0.3, 0.3, 0.3)
  love.graphics.rectangle("fill", window.width - width, 0, width, window.height)
  love.graphics.setColor(0.2, 0.2, 0.2)
  love.graphics.rectangle("fill", window.width - width-5, 0, 5, window.height)
  love.graphics.setColor(1,1,1)

  love.graphics.print("Attributes", window.width - width + 20, 10, 0, scale)
  love.graphics.print("Press enter to finalise changes", window.width - width +10, window.height - 40, 0, scale)
  
  local j = 0
  if #self.buttons ~= 0 then
    for i, button in ipairs(self.buttons) do
      
      
      if button.selected then
        love.graphics.setColor(0.22,0.22,0.22)
        love.graphics.rectangle("fill", button.x-5, button.y, button.width, button.height)
        love.graphics.setColor(0,1,1)
      else
        love.graphics.setColor(0.25,0.25,0.25)
        love.graphics.rectangle("fill", button.x-5, button.y, button.width, button.height)
        love.graphics.setColor(1,1,1)
      end
      
      if button.value == true then
        love.graphics.setColor(0.5,1,0.5)
      elseif button.value == false then
        love.graphics.setColor(1,0.5,0.5)
      elseif type(button.value) == "number" then
        love.graphics.setColor(0.3,0.5,1)
      elseif type(button.value) == "string" then
        love.graphics.setColor(1,1,0.5)
      else
        love.graphics.setColor(1,1,1)
      end
      
      
      love.graphics.print(tostring(button.value), button.x, button.y+1, 0, scale)
      
      love.graphics.setColor(0.3, 0.3, 0.3)
      love.graphics.rectangle("fill", button.x + button.width-5, button.y, love.graphics.getWidth() - (button.x + button.width), button.height)
      love.graphics.setColor(1,1,1)
      
      
      --Squash the next if it won't fit on screen
      local nameScaleX = math.min((window.width - (button.x + button.width+2)) / (font:getWidth(lowercase(button.name))), scale)
      
      
      love.graphics.print(lowercase(button.name), button.x + button.width, button.y+2, 0, nameScaleX, scale)
      j = j + 1
    end
  else
    love.graphics.setColor(1,1,1)
    love.graphics.print("No attributes", love.graphics.getWidth() - width + 10, 100)
  end
    
    
end
-- From https://stackoverflow.com/questions/29573473/lua-pattern-replace-uppercase-letters
-- I was not expecting to find the thing I exactly needed
function lowercase(str)
  return (str:gsub("%u", function(c) return ' ' .. c:lower() end))
end
  
  
function pointInRect(rect, x, y)

  if rect.x <= x and x <= rect.x + rect.width  
  and rect.y <= y and y <= rect.y + rect.height then
    return true
  else
    return false
  end
end


