gmouse = mouse
local mouse = {}

function Builder:darkZoneCreation()
  if not (self.selectedObject and self.selectedObject.type == "darkZone") then

    if self.selectedObject then
      cout(self.selectedObject, self.selectedObject.type)
    end

    if clickedThisFrame.primary then 
      if self.activeDarkZone then
        self.lastPoint = self.activeDarkZone:addPoint(self.activeDarkZone:treatCoordinates(gmouse.worldX, gmouse.worldY))
      else
        self.lastPoint = {}
        local transformedX, transformedY = lockToPixels(gmouse.worldX, gmouse.worldY)

        self.activeDarkZone = DarkZone(transformedX, transformedY, nil, copyTable(self.storedAttributes))
        self.lastPoint = Vec2(gmouse.worldX, gmouse.worldY)
      end
    else

      if (self.lastPoint ~= {} and self.lastPoint and love.mouse.isDown(1)) then
        local distance = 30
        if vector.getMagnitudeSqrd(Vec2(gmouse.worldX, gmouse.worldY) - self.lastPoint) > (distance/camera.zoom)^2 and self.activeDarkZone then
          self.lastPoint = self.activeDarkZone:addPoint(self.activeDarkZone:treatCoordinates(gmouse.worldX, gmouse.worldY))
        end
      end
    end
  end


  if clickedThisFrame.secondary then
    if self.activeDarkZone then
      if self.activeDarkZone.numberOfPoints > 1 then
        self.activeDarkZone:removePoint()
      else
        self.activeDarkZone:delete()
      end
    end
  end


  if love.keyboard.isScancodeDown("return") then
    if self.activeDarkZone or (self.selectedObject and self.selectedObject.type == "darkZone") and not love.mouse.isDown(1) then
      if self.activeDarkZone and self.activeDarkZone.numberOfPoints > 2 then
        self.activeDarkZone:finalise()
        self.activeDarkZone = false
      elseif self.selectedObject then
        self.selectedObject:remake()
      end
      --currentLevel:saveLevel("autosaves/autosave "..os.date('%Y-%m-%d %H-%M-%S'))
    end
  end

  if not love.mouse.isDown(1) then
    self.selectedVertex = nil
  end
end



function Builder:polygonCreation()
  if not (self.selectedObject and self.selectedObject.type ~= "polygon") then

    if clickedThisFrame.primary then 
      if self.activePolygon then
        self.lastPoint = self.activePolygon:addPoint(self.activePolygon:treatCoordinates(gmouse.worldX, gmouse.worldY))
      else
        self.lastPoint = {}
        local transformedX, transformedY = lockToPixels(gmouse.worldX, gmouse.worldY)

        self.activePolygon = LevelPolygon(transformedX, transformedY, nil, copyTable(self.storedAttributes))
        self.lastPoint = Vec2(gmouse.worldX, gmouse.worldY)
      end
    else

      if (self.lastPoint ~= {} and self.lastPoint and love.mouse.isDown(1)) then
        local distance = 30
        if vector.getMagnitudeSqrd(Vec2(gmouse.worldX, gmouse.worldY) - self.lastPoint) > (distance/camera.zoom)^2 and self.activePolygon then
          self.lastPoint = self.activePolygon:addPoint(self.activePolygon:treatCoordinates(gmouse.worldX, gmouse.worldY))
        end
      end
    end
  end


  if clickedThisFrame.secondary then
    if self.activePolygon then
      if self.activePolygon.numberOfPoints > 1 then
        self.activePolygon:removePoint()
      else
        self.activePolygon:delete()
      end
    end
  end


  if love.keyboard.isScancodeDown("return") then
    if self.activePolygon or (self.selectedObject and self.selectedObject.type == "levelPolygon") and not love.mouse.isDown(1) then
      if self.activePolygon and self.activePolygon.numberOfPoints > 2 then
        self.activePolygon:finalise()
        self.activePolygon = false
      elseif self.selectedObject then
        self.selectedObject:remake()
      end
      --currentLevel:saveLevel("autosaves/autosave "..os.date('%Y-%m-%d %H-%M-%S'))
    end
  end

  if not love.mouse.isDown(1) then
    self.selectedVertex = nil
  end
end


function Builder:propCreation()
  if not self.selectedObject then

    local propTexture = self.storedAttributes.textureName
    local propImage = self:getPropImage()


    local translatedX, translatedY = nil, nil
    if self:isModeProp() or self.mode == "flow" then
      translatedX, translatedY = lockToPixels(gmouse.worldX, gmouse.worldY, "small", 
        propImage:getWidth()%2 == 1,
        propImage:getHeight()%2 == 1)
    end




    if clickedThisFrame.primary then

      local attributes = copyTable(self.storedAttributes)

      if self.mode == "prop" then
        Prop(translatedX, translatedY, attributes)
      elseif self.mode == "logicProp" then
        LogicProp(translatedX, translatedY, attributes)
      elseif self.mode == "talkative" then
        Talkative(translatedX, translatedY, attributes)
      elseif self.mode == "door" then
        Door(translatedX, translatedY, attributes)
      elseif self.mode == "slidingDoor" then
        SlidingDoor(translatedX, translatedY, attributes)
      elseif self.mode == "brazier" then
        Brazier(translatedX, translatedY, attributes)
      elseif self.mode == "rainEmitter" then
        RainEmitter(translatedX, translatedY, attributes)
      elseif self.mode == "light" then
        LightEmitter(translatedX, translatedY, attributes)
      elseif self.mode == "flow" then
        Flow(translatedX, translatedY, attributes)
      elseif self.mode == "directionStone" then
        DirectionStone(translatedX, translatedY, attributes)
      end
    end
  end
end


function Builder:grassCreation()
  local translatedX, translatedY = lockToPixels(gmouse.worldX, gmouse.worldY, "small")


  if clickedThisFrame.primary then
    if self.activeGrass then
      self.activeGrass:addPoint(translatedX, translatedY)
    else
      self.activeGrass = Grass(translatedX, translatedY)
    end
  end


  if clickedThisFrame.secondary then
    if self.activeGrass then
      if #self.activeGrass.points > 1 then
        self.activeGrass:removePoint()
      else
        self.activeGrass.remove = true
        self.activeGrass = nil
        flushDrawTable()
      end
    end
  end

  if love.keyboard.isScancodeDown("return") then
    if self.activeGrass then
      self.activeGrass.finalised = true
      self.activeGrass = nil
      --currentLevel:saveLevel("autosaves/autosave "..os.date('%Y-%m-%d %H-%M-%S'))
    end
  end

  if self.activeGrass then
    self.activeGrass.nextPoint.x, self.activeGrass.nextPoint.y = self.activeGrass:getNextPoint(translatedX, translatedY)
  end
end


function Builder:fireflyCreation()
  local translatedX, translatedY = lockToPixels(gmouse.worldX, gmouse.worldY, "small")
  local distance = 50
  local drawDistance = 10
  self.mouseLastPos = self.mouseLastPos or gmouse
  if clickedThisFrame.primary then
    self.mouseLastPos = Vec2(gmouse.worldX, gmouse.worldY)
    Firefly(translatedX, translatedY)

  end
  local k = 0.5
  if love.mouse.isDown(1) and vector.getMagnitudeSqrd(vector.subtraction(Vec2(gmouse.worldX, gmouse.worldY), self.mouseLastPos)) >= drawDistance^2 then
    self.mouseLastPos = Vec2(gmouse.worldX, gmouse.worldY)
    Firefly(translatedX + love.math.random(-distance, distance)*k, translatedY + love.math.random(-distance, distance)*k)
  end
end


function Builder:boxCreation()
  local translatedX, translatedY = lockToPixels(gmouse.worldX, gmouse.worldY, "small")
  if clickedThisFrame.primary then
    PhysicsCube(translatedX, translatedY)
  end
end


function Builder:sqeezeCreation()
  local x, y = lockToPixels(gmouse.worldX, gmouse.worldY, "small")
  
  if not self.selectedObject then
    if not self.activeSqueeze then
      if clickedThisFrame.primary then
        self.activeSqueeze = Squeeze(x, y)
      end
    else
      if clickedThisFrame.primary then
        
        self.activeSqueeze:addPoint(x, y)
        
      elseif clickedThisFrame.secondary then
        
        if #self.activeSqueeze.points == 1 then
          self.activeSqueeze:delete()
          self.activeSqueeze = nil
        else
          self.activeSqueeze:removePoint()
        end
        
      elseif love.keyboard.isScancodeDown("return") then
        
        self.activeSqueeze.finalised = true
        self.activeSqueeze = nil
        
      end
    end
  end
end
