local builderMenu = {}
builderMenu.selectedTab = 1

builder = nil

function builderMenu:newTab(name, image)

  local newTab = {}
  newTab.name = name
  newTab.image = image
  newTab.profiles = {}

  table.insert(self.tabs, newTab)
end


function builderMenu:newProfile(name, tabName, image, pressFunction, arguments)

  if type(tabName) ~= "string" then error("Tab name was not a string", 2)     end
  if type(name)    ~= "string" then error("Profile name was not a string", 2) end

  local profile = {}

  profile.name = name
  profile.image = image
  profile.pressFunction = pressFunction
  if arguments then
    profile.arguments = {builder, unpack(arguments)}
  end

  local tab

  for i, v in ipairs(self.tabs) do
    if v.name == tabName then
      tab = v
      break
    end
  end

  if not tab then
    error("Could not find tab", 2)
  end


  table.insert(tab.profiles, profile)

  return profile
end


function builderMenu:populate(parentBuilder)
  
  builder = parentBuilder

  self.tabs = {}
  self:newTab("Props", texture.placeholder)
  self:newTab("Environment", texture.placeholder)
  self:newTab("Characters", texture.placeholder)
  self:newTab("Tools", texture.placeholder)

  
--                name         tabName  image                   pressFunction    {arguments}
  self:newProfile("Bookshelf", "Props", texture.bookshelf,      Builder.setMode, {"prop", {textureName = "bookshelf"}})
  self:newProfile("Barrel",    "Props", texture.barrel,         Builder.setMode, {"prop", {textureName = "barrel"}})
  self:newProfile("Table",     "Props", texture.table,          Builder.setMode, {"prop", {textureName = "table"}})
  self:newProfile("Mug",       "Props", texture.mug,            Builder.setMode, {"prop", {textureName = "mug"}})
  self:newProfile("Kettle",    "Props", texture.kettle,         Builder.setMode, {"prop", {textureName = "kettle"}})
  self:newProfile("Stove",     "Props", texture.stove,          Builder.setMode, {"prop", {textureName = "stove"}})
  self:newProfile("Pot",       "Props", texture.pot,            Builder.setMode, {"prop", {textureName = "pot"}})
  self:newProfile("Backpack",  "Props", texture.backpack,       Builder.setMode, {"prop", {textureName = "backpack"}})
  self:newProfile("Grapple",   "Props", texture.grappling_hook, Builder.setMode, {"prop", {textureName = "grappling_hook"}})
  self:newProfile("Rope Coil", "Props", texture.rope_coil,      Builder.setMode, {"prop", {textureName = "rope_coil"}})
  
  self:newProfile("Door",         "Props", texture.doors.wooden.open,   Builder.setMode, {"door"})
  self:newProfile("Sliding Door", "Props", texture.doors.wooden.closed, Builder.setMode, {"slidingDoor"})
  self:newProfile("Box",          "Props", texture.stoneBox,            Builder.setMode, {"box"})
  
  self:newProfile("Brazier",        "Props", texture.brazier_lit,       Builder.setMode, {"brazier", {textureName = "brazier", animation = "brazier_lit"}})
  self:newProfile("DirectionStone", "Props", texture.direction_stone,   Builder.setMode, {"directionStone", {name = "linearRepeatEnd"}})
  
  
  
  self:newProfile("Stone",      "Environment", texture.icon_stone,           Builder.setMode, {"polygon", {textureName = "stone"}})
  self:newProfile("Stone Back", "Environment", texture.icon_backgroundStone, Builder.setMode, {"polygon", {textureName = "stone", isBackground = true, depth = -40}})
  self:newProfile("Foreground", "Environment", texture.icon_stone,           Builder.setMode, {"polygon", {textureName = "stone", collidable = false, fadeWhenBehind = true}})
  self:newProfile("Wood",       "Environment", texture.icon_wood,            Builder.setMode, {"polygon", {textureName = "wood"}})
  self:newProfile("Wood Back",  "Environment", texture.icon_backgroundWood,  Builder.setMode, {"polygon", {textureName = "wood", isBackground = true, depth = -30}})
  self:newProfile("Moss",       "Environment", texture.moss,               Builder.setMode, {"polygon", {textureName = "moss", priorityDraw = true, superTexture = false, collidable = false}})
  self:newProfile("Water",      "Environment", texture.water,              Builder.setMode, {"polygon", {textureName = "water", wetness = 10, collidable = false}})
  
  
  self:newProfile("Grass",        "Environment", texture.grassFlower[1], Builder.setMode, {"grass"})
  self:newProfile("Fireflies",    "Environment", texture.firefly,        Builder.setMode, {"fireflies"})
  self:newProfile("Rain Emitter", "Environment", texture.icon_rain  ,    Builder.setMode, {"rainEmitter"})
  self:newProfile("Single Drip",  "Environment", texture.raindrop,       Builder.setMode, {"rainEmitter", {amount = 0.5, spreadX = 0, spreadY = 0}})
  self:newProfile("Vine",         "Environment", texture.vine,       Builder.setMode, {"prop", {textureName = "vine"}})
  self:newProfile("long Vine",    "Environment", texture.long_vine,  Builder.setMode, {"prop", {textureName = "long_vine"}})
  self:newProfile("fork Vine",    "Environment", texture.fork_vine,  Builder.setMode, {"prop", {textureName = "fork_vine"}})
  self:newProfile("short Vine",   "Environment", texture.short_vine, Builder.setMode, {"prop", {textureName = "short_vine"}})
  
  
  
  self:newProfile("Prop",      "Tools", texture.icon_prop,  Builder.setMode, {"prop", {textureName = "icon_prop"}})
  self:newProfile("Light",     "Tools", texture.icon_prop,  Builder.setMode, {"light", {textureName = "icon_prop"}})
  self:newProfile("Logic",     "Tools", texture.logic_prop, Builder.setMode, {"logicProp", {textureName = "logic_prop"}})
  self:newProfile("Talkative", "Tools", texture.icon_talkative, Builder.setMode, {"talkative", {textureName = "icon_talkative"}})
  
  self:newProfile("Dark Zone", "Tools", texture.icon_stone, Builder.setMode, {"darkZone"})
  
  self:newProfile("Squeeze",   "Tools", texture.icon_prop, Builder.setMode,  {"squeeze"})

  
  self:newProfile("Voltehre", "Characters", texture.voltehre_idle, Builder.setMode, {"talkative", {textureName = "voltehre_idle"}})
  self:newProfile("Cordelia", "Characters", texture.cordelia_idle, Builder.setMode, {"talkative", {textureName = "cordelia_idle"}})
  self:newProfile("Curator",  "Characters", texture.curator_idle,  Builder.setMode, {"talkative", {textureName = "curator_idle", requiresFacing = true}})
  
  self:newProfile("Flow (Unfinished content)",     "Characters", texture.flow,          Builder.setMode, {"flow"})
  
end


local function getButtonShape(centreX, centreY)
  local buttonWidth = window.width/9.5
  local buttonHeight = window.height/5


  local x = centreX - buttonWidth/2
  local y = centreY - buttonHeight/2
  return x, y, buttonWidth, buttonHeight
end


function builderMenu:getButtonPosition(i)
  i = i - 1
  local columns = 7
  local row = math.floor(i/columns)
  local column = i%columns

  local separationWidth = self.width/columns
  local separationHeight = select(4, getButtonShape(0,0)) + font:getHeight()*2+ 3

  local x = separationWidth * column + self.x + separationWidth/2
  local y = separationHeight * row + self.y + separationHeight/2
  return x, y
end


function builderMenu:changeTab(newTab)
  

  self.selectedTab = newTab or self.selectedTab + 1
  
end


function builderMenu:update()
  
  
  if controls.menuLeft.thisFrame then
    self.selectedTab = self.selectedTab - 1
  elseif controls.menuRight.thisFrame then
    self.selectedTab = self.selectedTab + 1
  end
  
  self.selectedTab = ((self.selectedTab-1) % #self.tabs) + 1
  
  
  

  local tab = self.tabs[self.selectedTab]


  self.x = window.width/10
  self.y = window.height/10

  self.width = window.width - window.width/5
  self.height = window.height - window.height/5


  for i, profile in ipairs(tab.profiles) do

    if clickedThisFrame.primary then
      local x, y, width, height = self:getButtonPosition(i)
      x, y, width, height = getButtonShape(x, y)

      if pointInRect({x = x, y = y, width = width, height = height}, love.mouse.getX(), love.mouse.getY()) then

        if profile.arguments then
          profile.pressFunction(unpack(profile.arguments))
        else
          profile.pressFunction()
        end
        
        local positiveSound = sound.effect.ui.select:clone()
        positiveSound:play()
        
        builder:hideMenu()

      end
    end

  end

end


function builderMenu:draw()
  local tab = self.tabs[self.selectedTab]
  local xGap = window.width/10
  local yGap = window.height/10
  local titleScale = self.y/(font:getHeight() + 4)

  love.graphics.setColor(0.5, 0.5, 0.5)


  love.graphics.rectangle("fill", self.x, 0, font:getWidth(tab.name)*titleScale + 20, self.y)
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
  
  love.graphics.setLineWidth(10)
  love.graphics.setColor(0.45, 0.45, 0.45)
  love.graphics.rectangle("line", self.x, self.y, self.width, self.height)
  love.graphics.rectangle("line", self.x, 0, font:getWidth(tab.name)*titleScale + 20, self.y)

  --Draw tab title
  
  love.graphics.setColor(1, 1, 1)
  love.graphics.print(tab.name, self.x + 10, 5, 0, titleScale)
  
  
  love.graphics.displayCentredKey(self.x - 25, self.y/2, controls.menuLeft.name, 1.5)
  love.graphics.displayCentredKey(self.x + font:getWidth(tab.name)*titleScale + 20 + 25, self.y/2, controls.menuRight.name, 1.5)


  

  local labelString 

  for i, profile in ipairs(tab.profiles) do
    local x, y, width, height = self:getButtonPosition(i)



    x, y, width, height = getButtonShape(x, y)


    if pointInRect({x = x, y = y, width = width, height = height}, love.mouse.getX(), love.mouse.getY()) then
      love.graphics.setColor(0.7, 0.7, 0.7)
    else
      love.graphics.setColor(0.6, 0.6, 0.6)
    end

    love.graphics.rectangle("fill", x, y, width, height)
    love.graphics.setColor(0.45, 0.45, 0.45)
    love.graphics.setLineWidth(5)
    love.graphics.rectangle("line", x, y, width, height)
    
    love.graphics.setColor(1,1,1)


    if type(profile.image) == "table" then
      local scale
      local image 
      
      if profile.image.main:getWidth()/width > profile.image.main:getHeight()/height then
        scale = (width/(profile.image.main:getWidth()-2))*0.9
      else
        scale = (height/(profile.image.main:getHeight()-2))*0.9
      end
      local drawX = x + width/2 - profile.image.main:getWidth()/2*scale
      local drawY = y + height/2 - profile.image.main:getHeight()/2*scale
      
      love.graphics.setColor(1,1,1)
      love.graphics.draw(profile.image.main, drawX, drawY, 0, scale, scale)
      love.graphics.setColor(0, 0, 0, scene.lightHarshness)
      if profile.image.shadow then
        love.graphics.draw(profile.image.shadow, x, drawY, 0, scale, scale)
      end
    else
      local scale
      
      if profile.image:getWidth()/width > profile.image:getHeight()/height then
        scale = (width/(profile.image:getWidth()-2))*0.9
      else
        scale = (height/(profile.image:getHeight()-2))*0.9
      end
      local drawX = x + width/2 - profile.image:getWidth()/2*scale
      local drawY = y + height/2 - profile.image:getHeight()/2*scale
      love.graphics.draw(profile.image, drawX, drawY, 0, scale, scale)
    end
    love.graphics.setColor(1,1,1)



    local scaleX = 2
    if font:getWidth(profile.name)*2 > width then
      scaleX = width / (font:getWidth(profile.name))
    end

    love.graphics.print(profile.name, x + (width/2) - font:getWidth(profile.name)/2*scaleX , y + height, 0, scaleX, 2)
    
    
    if pointInRect({x = x, y = y, width = width, height = height}, love.mouse.getX(), love.mouse.getY()) then
      labelString = profile.arguments[2]:gsub("^%l", string.upper)
    end
    
  end
  
  if labelString then
    love.graphics.setColor(0.2, 0.2, 0.2, 0.5)
    love.graphics.rectangle("fill", love.mouse.getX()+5, love.mouse.getY()+5, font:getWidth(labelString)*1.5+10, font:getHeight()*1.5+10)
    love.graphics.setColor(1,1,1)
    love.graphics.print(labelString, love.mouse.getX() + 10, love.mouse.getY() + 10, 0, 1.5)
  end
  
  
  love.graphics.setLineWidth(1)
end

return builderMenu