Builder = Entity:extend()

--Patch job - FIX PLEASE
-- NO
gmouse = mouse


require "builderCreations"


local builderMenu



function Builder:new(x, y)
  Builder.super.new(self, x, y)
  self.type = "builder"
  self.id = nextEntityID
  nextEntityID = nextEntityID + 1

  self.selectedObject = nil
  self.selectedVertex = nil -- key of the selected polygon
  self.storedAttributes = {}
  self.showBuilderMenu = false

  self.mode = "polygon"


  builderMenu = require "builderMenu"
  builderMenu:populate(self)

end




function Builder:setMode(mode, attributes)
  self.mode = mode
  self.storedAttributes =  attributes or {}
  if self.activePolygon then
    self.activePolygon:delete()
  end
  if self.activeDarkZone then
    self.activeDarkZone:delete()
  end
end


function love.wheelmoved(x, y)
  if builderMode and not builder.showBuilderMenu then
    camera.zoom = camera.zoom*1.07^y
  end
end


function Builder:showMenu()
  self.showBuilderMenu = true
end  

function Builder:hideMenu()
  self.showBuilderMenu = false
end  

function Builder:toggleMenu()
  self.showBuilderMenu = not self.showBuilderMenu
end


function Builder:isModeSelectable()
  return self.mode == "polygon" or "darkZone" or self:isModeProp() or self.mode == "squeeze"
end

function Builder:isModeProp()
  return 
  self.mode == "prop" or 
  self.mode == "button" or 
  self.mode == "talkative" or 
  self.mode == "logicProp" or 
  self.mode == "door" or 
  self.mode == "slidingDoor" or 
  self.mode == "brazier" or 
  self.mode == "rainEmitter" or 
  self.mode == "light" or 
  self.mode == "directionStone"
end


function Builder:getPropImage()

  local propTexture = self.storedAttributes.textureName
  if self.mode == "door" then
    return texture.doors.wooden.closed
  elseif self.mode == "slidingDoor" then
    return texture.doors.wooden.closed
  elseif self.mode == "rainEmitter" then
    return texture.icon_rain
  elseif self.mode == "directionStone" then
    return texture.direction_stone
  else
    if texture[propTexture].main then
      return texture[propTexture].main
    else
      return texture[propTexture]
    end
  end
end



--All level builder code should be contained within this function

function Builder:update(dt)

  if self.showBuilderMenu then
    builderMenu:update()
  else
    local moveSpeed = 800


    if love.keyboard.isDown("r") then
      player1.body:setPosition(gmouse.worldX, gmouse.worldY)
    end


    if love.keyboard.isScancodeDown("lshift") then
      moveSpeed = 1600
    end

    if love.keyboard.isScancodeDown("right","d") then
      self.x = self.x + moveSpeed*dt/camera.zoom
    end
    if love.keyboard.isScancodeDown("left", "a") then
      self.x = self.x - moveSpeed*dt/camera.zoom
    end
    if love.keyboard.isScancodeDown("up", "w") then
      self.y = self.y - moveSpeed*dt/camera.zoom
    end
    if love.keyboard.isScancodeDown("down", "s") then
      self.y = self.y + moveSpeed*dt/camera.zoom
    end

    --creation loop
    
    if self.mode == "polygon" then

      self:polygonCreation()

    elseif self.mode == "darkZone" then

      self:darkZoneCreation()

    elseif self:isModeProp() or self.mode == "flow" then
      
      self:propCreation()
      
    elseif self.mode == "grass" then
      
      self:grassCreation()

    elseif self.mode == "box" then
      
      self:boxCreation()
      
    elseif self.mode == "fireflies" then
      
      self:fireflyCreation()
      
    elseif self.mode == "squeeze" then
      
      self:sqeezeCreation()
      
    end


    if (not self.activePolygon) and (not self.activeDarkZone) and clickedThisFrame.secondary then
      if self:isModeSelectable() and (not self.selectedObject) then
        local mode = self.mode
        if self.mode == "polygon" then
          mode = "levelPolygon"
        elseif self:isModeProp() then
          mode = "prop"
        end        
        for i, object in ipairs(drawTable) do

          if object.type == mode or 
             (mode == "prop" and object.isProp) or 
             (mode == "squeeze" and object.type == "squeeze") then
               
            if object:checkSelect() then
              self.selectedObject = object
              object.selected = true
              break
            end
          end
        end 
      end
    end


    if love.keyboard.isScancodeDown("return") and not love.mouse.isDown(1) then
      if self.selectedObject then
        self.selectedObject.selected = false
        self.selectedObject = nil
--        currentLevel:saveLevel("autosaves/autosave "..os.date('%Y-%m-%d %H-%M-%S'))
      end
    end



    if (self:isModeSelectable()) and self.selectedObject then
      attributeEditor:update(self.selectedObject)
    end
  end
end


function Builder:draw()
  if not (screenshotMode or self.showBuilderMenu) then
    local offsetY = false
    local offsetX = false

    local propTexture = self.storedAttributes.textureName
    local propImage   = self:getPropImage()


    if self:isModeProp() or self.mode == "flow" then
      offsetX = propImage:getWidth()%2 == 1
      offsetY = propImage:getHeight()%2 == 1
    end



    local translatedX, translatedY = lockToPixels(gmouse.worldX, gmouse.worldY, "small", offsetX, offsetY)
    if self:isModeProp() and not self.selectedObject then
      if texture[propTexture].main then
        love.graphics.setColor(1,1,1,0.7)
        blue.graphics.drawCentredSet(texture[propTexture], translatedX, translatedY, 0, smallTextureScale, smallTextureScale)
      else
        love.graphics.setColor(1,1,1,0.7)
        blue.graphics.drawCentred(propImage, translatedX, translatedY, 0, smallTextureScale, smallTextureScale)
      end

    elseif self.mode == "grass" then
      if not self.activeGrass then
        love.graphics.setColor(1,1,1, 0.7)
        blue.graphics.drawCentred(texture.grass[1], translatedX, translatedY, 0, smallTextureScale, smallTextureScale)
      end
    elseif self.mode == "box" then
      love.graphics.setColor(1,1,1, 0.7)
      blue.graphics.drawCentred(texture.stoneBox, translatedX, translatedY, 0, smallTextureScale, smallTextureScale)
    elseif self.mode == "flow" then

      love.graphics.setColor(1,1,1, 0.7)
      blue.graphics.drawCentred(texture.flow, translatedX, translatedY, 0, smallTextureScale, smallTextureScale)

    end
    if self.selectedObject and self:isModeSelectable() then
      attributeEditor:draw()
    end
  end

  if self.showBuilderMenu then
    builderMenu:draw()
  end
end

