DarkZone = LevelPolygon:extend()

local passes = require "rendering"

DarkZone.getStickTracks = nil


function DarkZone:new(x, y, inputPoints, attributes)
  
  Polygon.super.new(self, x, y)
  self.type = "darkZone"


  self.attributes = attributes or {}
  
  
  createAttribute(self.attributes, "red", 1)
  createAttribute(self.attributes, "green", 1)
  createAttribute(self.attributes, "blue", 1.1)
  createAttribute(self.attributes, "matchOutside", false)
  createAttribute(self.attributes, "hiddenOutside", false)
  createAttribute(self.attributes, "intensity", 0.6)
  createAttribute(self.attributes, "protected", false)
  createAttribute(self.attributes, "transparency", 1)
  createAttribute(self.attributes, "drawPriority", 0)

  self.attributes.ignore = false -- It's not recommended that you create an attribute manually, but I always want loaded polygons to not be ignored
  
  if inputPoints then
    self.points = inputPoints
    self.finalised = true
  else
    self.points = {}
    self.finalised = false
    self.points[1] = Vec2(x, y)
    self.numberOfPoints = 1
  end
  
  
  self.selected = false
  table.insert(darkZoneTable, self)
  table.insert(drawTable, self)
  
  self.currentTransparency = self.attributes.transparency or 1
  
  
end


function DarkZone:setAttributes()
  sortDarkZoneTable()
end


function DarkZone:delete()
  
  if not self.finalised then
    builder.activeDarkZone.remove = true
    builder.activeDarkZone = nil
    
  elseif not self.remove then
    
    if builder.selectedDarkZone == self then
      builder.selectedDarkZone = nil
    end
    self.remove = true
    
    flushDrawTable()
    
    for i, darkZone in ipairs(darkZoneTable) do
      if darkZone == self then
        table.remove(darkZoneTable, i)
      end
    end

  end
end


function DarkZone:updateBuilder()

  if builder.mode == "darkZone" then
    if self.finalised then
      if self.selected then
        if love.keyboard.isScancodeDown("delete") then
          local selectedPoint = self:selectPoint(mouse.worldX, mouse.worldY, self.points)
          if selectedPoint then
            self:removePoint(selectedPoint)
          end
        end
      end
      
      local inThisDarkZone
      if self.body == nil or (not self.body:isDestroyed()) then
        inThisDarkZone = self:checkInside(mouse.worldX, mouse.worldY)
      end

      if inThisDarkZone then
        --If mouse is over darkZone
        if love.keyboard.isScancodeDown("delete") and not self.selected then
          
          
          
          if not self.attributes.protected then
            self:delete()
          end
        end
      end
      
      if self.selected then
        self:modifyPoints(inThisDarkZone)
      end
    end
  end

  if self.selected then
    --Selected but not finalised
    builder.selectedDarkZone = self
  elseif builder.selectedDarkZone == self then
    builder.selectedDarkZone = nil
  end
end


function sortDarkZoneTable()
  table.sort(darkZoneTable, 
      function(k1, k2)
        --True if k2 should be in front of k1
        return k2.attributes.drawPriority > k1.attributes.drawPriority
      end
  )
end


function DarkZone:finalise()
  self.finalised = true
  local newPointFormat = {}
  for i, point in ipairs(self.points) do
    table.insert(newPointFormat, point.x)
    table.insert(newPointFormat, point.y)
  end
  self.points = {}
  self.points = newPointFormat


  currentLevel.darkZones[self] = self
  self:setAttributes()
  
end


function DarkZone:checkInside(x, y)
  if self.finalised then
    return isPointInPolygon(x, y, self.points)
  else
    return false
  end
end


function DarkZone:update(dt)
  
  if self.attributes.matchOutside then
    
    passes:getAmbient()
    
  end
  
  
  if self.attributes.hiddenOutside then
    local fadeSpeed = 4
    if self:checkInside(player1.body:getPosition()) then
      
      self.currentTransparency = math.max(self.currentTransparency + dt * fadeSpeed, 0)
      
    else
      
      self.currentTransparency = math.min(self.currentTransparency - dt * fadeSpeed, 1)
      
    end
  end
  
end


function DarkZone:draw()
  
  
end



function DarkZone:drawAmbient()

  if self.finalised then
    
    if builderMode then
      
      if self.attributes.matchOutside then
        local colour = passes:getAmbient()
        love.graphics.setColor(colour[1], colour[2], colour[3], 1)
        
      else
        love.graphics.setColor(
          self.attributes.red * self.attributes.intensity/4,
          self.attributes.green * self.attributes.intensity/4,
          self.attributes.blue * self.attributes.intensity/4, 
          1)
      end
    else
      if self.attributes.matchOutside then
        local colour = passes:getAmbient()
        love.graphics.setColor(colour[1], colour[2], colour[3], 1)
      else
        love.graphics.setColor(
          self.attributes.red * self.attributes.intensity/4,
          self.attributes.green * self.attributes.intensity/4,
          self.attributes.blue * self.attributes.intensity/4, 
          self.currentTransparency)
      end
    end
    
    blue.graphics.polygon("fill", self.points)
  end
end



