directionParticles = {}
local textureWidth
local textureHeight

function createDirectionParticleGroup()
  directionParticles.members = {}
  directionParticles.type = "directionParticles"
  directionParticles.spriteBatch = love.graphics.newSpriteBatch(texture.direction_particle, 1000)
  textureWidth  = texture.direction_particle:getWidth()
  textureHeight = texture.direction_particle:getHeight()
  table.insert(drawTable, directionParticles)
end


function directionParticles:updatePaused(dt)
  for i, directionParticle in ipairs(self.members) do
    directionParticle:update(dt, i)
  end
end


function directionParticles:draw()
  
  self.spriteBatch:clear()
  
  for i, directionParticle in ipairs(self.members) do
    self.spriteBatch:setColor(1, 1, 1, directionParticle.opacity)
    self.spriteBatch:add(directionParticle.x, directionParticle.y, directionParticle.angle, smallTextureScale, smallTextureScale, textureWidth/2, textureHeight/2)
  end
  
  love.graphics.setColor(1,1,1)
  blue.graphics.draw(directionParticles.spriteBatch, 0, 0)
end




DirectionParticle = Entity:extend()

function DirectionParticle:new(x, y, directionVector, life)
  DirectionParticle.super.new(self, x, y)
  self.type = "directionParticle"
  self.opacity = 1
  self.directionVector = directionVector
  self.angle = math.rad(love.math.random(360))
  self.life = life or 8
  self.speed = 0
  self.rotationVelocity = math.rad((love.math.random()*2-1)*0.4)
  table.insert(directionParticles.members, self)
end



function DirectionParticle:update(dt, index)
  self.type = "directionParticle"
  --self.opacity = self.opacity - dt
  
  self.life = self.life - dt
  
  if self.life <= 0 then
    table.remove(directionParticles.members, index)
  else
    self.speed = self.speed + dt*0.05
    self.x = self.x + self.directionVector.x*self.speed
    self.y = self.y + self.directionVector.y*self.speed
    self.angle = self.angle + self.rotationVelocity
    self.opacity = math.min(self.life*0.5, 1)
    
  end
end





