Flow = Entity:extend()


require "entities/flowLevelPolygon"


local function getPolygonCirclePieces(radius, thickness, circleEnds)
  
  
  local polygons = {}
  local numberOfPoints = 40
  
  for i = 1, #circleEnds, 2 do
  
    local a1 = circleEnds[i]
    local a2 = circleEnds[i+1]
    
    local polygon = {}
    local angle = math.rad(a2 - a1)
    
    do
      local arcLength = angle * radius
      
      table.insert(polygon, math.cos(math.rad(a1))*radius)
      table.insert(polygon, math.sin(math.rad(a1))*radius)
      local separationAngle = angle/(numberOfPoints+1)
      for i = 1, numberOfPoints do
        table.insert(polygon, math.cos(math.rad(a1)+separationAngle*i)*radius)
        table.insert(polygon, math.sin(math.rad(a1)+separationAngle*i)*radius)
      end
      table.insert(polygon, math.cos(math.rad(a2))*radius)
      table.insert(polygon, math.sin(math.rad(a2))*radius)
    end
    
    
    do
      local arcLength = angle * (radius - thickness)
      
      table.insert(polygon, math.cos(math.rad(a2))*(radius - thickness))
      table.insert(polygon, math.sin(math.rad(a2))*(radius - thickness))
      local separationAngle = angle/(numberOfPoints+1)
      for i = 1, numberOfPoints do
        table.insert(polygon, math.cos(math.rad(a2)-separationAngle*i)*(radius - thickness))
        table.insert(polygon, math.sin(math.rad(a2)-separationAngle*i)*(radius - thickness))
      end
      table.insert(polygon, math.cos(math.rad(a1))*(radius - thickness))
      table.insert(polygon, math.sin(math.rad(a1))*(radius - thickness))
    end
    table.insert(polygons, polygon)
    
  end


  return polygons
end



function Flow:new(x, y)
  Prop.super.new(self, x, y)
  table.insert(drawTable, self)
  self.type = "flow"
  
  self.timer = 60
  
  -- Note - should set up some sort of draw heirarchy for pieces that overlap
  local polygons
  
  self:createRing(-2,  750*smallTextureScale, 500*smallTextureScale, {0, 60, 120, 180, 240, 300})
  self:createRing(2,  1000*smallTextureScale, 250*smallTextureScale, {0, 110, 120, 230, 240, 350})
 --[[ self:createRing(1 , 1350*smallTextureScale, 100*smallTextureScale, {0, 50, 60, 110, 120, 170, 180, 230, 240, 290, 300, 350})
  self:createRing(-1, 1600*smallTextureScale, 250*smallTextureScale, {0, 50, 60, 110, 120, 170, 180, 230, 240, 290, 300, 350})
  self:createRing(1,  1700*smallTextureScale, 100*smallTextureScale, {0, 50, 60, 110, 120, 170, 180, 230, 240, 290, 300, 350})]]
  
  
end





function Flow:createRing(speed, radius, thickness, circleEnds)  
  local polygons = getPolygonCirclePieces(750*smallTextureScale, 500*smallTextureScale, {0, 60, 120, 180, 240, 300})
  for i, polygon in ipairs(polygons) do
    FlowLevelPolygon(speed, self, polygon)
  end  
end



function Flow:update(dt)
  
  self.timer = self.timer + dt
  
  local timerLength = 60
    
  if self.timer >= timerLength then
    sound.music.ethereal:play()
    self.timer = self.timer - timerLength
    
    local resetRadius = 1800*smallTextureScale
    
    
    local newDirection = vector.normalize(Vec2(player1.x, player1.y) - self)
    
    if newDirection.x == 0 and newDirection.y == 0 then
      newDirection.x = 1
      newDirection.y = 0
    end
    
    player1:setPosition((newDirection * resetRadius) + self)
    
  end
end


function Flow:draw()
  
  love.graphics.setColor(1,1,1)
  blue.graphics.drawCentred(texture.flow, self.x, self.y, 0, smallTextureScale)
  
end