Grass = Object:extend()

function Grass:new(x, y)
  self.type = "grass"
  self.points = {}
  self.points[1] = {}
  self.points[1].x = x
  self.points[1].y = y
  self.nextPoint = {}
  self.nextPoint.x = 0
  self.nextPoint.y = 0
  self.activeStage = currentLevel.stage
  self.flowerCoverage = 0.8
  table.insert(drawTable, self)
end

function Grass:addPoint(x, y)
  local thisPoint = #self.points+1
  local grassWidth = texture.grass[1]:getWidth()*smallTextureScale
  local deltaX = self.points[1].x - x
  
  self.points[thisPoint] = {}
  self.points[thisPoint].x = self.points[1].x - snapTo(deltaX, grassWidth)
  self.points[thisPoint].y = y 
end


function Grass:getNextPoint(x, y)
  local thisPoint = #self.points+1
  local grassWidth = texture.grass[1]:getWidth()*smallTextureScale
  local deltaX = self.points[1].x - x
  
  return self.points[1].x - snapTo(deltaX, grassWidth), y
end 


function Grass:removePoint(x, y)
  local thisPoint = #self.points
  self.points[thisPoint] = nil
end


local pointRadius = 10
function Grass:draw()
  local grassRNG = love.math.newRandomGenerator(0)
  local grassWidth = texture.grass[1]:getWidth()*smallTextureScale
  local grassHeight = texture.grass[1]:getHeight()*smallTextureScale
  
  
  local firstX, firstY = lockToPixels(self.points[1].x, self.points[1].y, "small")
  
  local grassTexture = nil
  if grassRNG:random() <= self.flowerCoverage then
    grassTexture = texture.grassFlower[grassRNG:random(1,3)]
  else
    grassTexture = texture.grass[grassRNG:random(1,3)]
  end
  
  love.graphics.setColor(1,1,1)
  
  --[[blue.graphics.drawCentredShaded(grassTexture,
    firstX, 
    firstY,
    0,
    smallTextureScale,
    smallTextureScale)]]
  

  love.graphics.setColor(1,1,1)
  for i, point in ipairs(self.points) do
    if i > 1 then
      local deltaX = self.points[i-1].x - point.x - grassWidth*0.5
      local deltaY = self.points[i-1].y - point.y
      local numberOfBlades = math.ceil(math.abs(deltaX/grassWidth))
      if self.points[2] then
        for j = 1, numberOfBlades-1 do
          local translatedX = self.points[i-1].x + j*grassWidth*(-deltaX/math.abs(deltaX))
          local translatedY = snapTo(self.points[i-1].y - (j/(numberOfBlades-1))*deltaY, smallTextureScale)
          local grassTexture = nil
          if grassRNG:random() <= self.flowerCoverage then
            grassTexture = texture.grassFlower[grassRNG:random(1,3)]
          else
            grassTexture = texture.grass[grassRNG:random(1,3)]
          end
          
          blue.graphics.drawCentredShaded(grassTexture,
            translatedX, 
            translatedY,
            0,
            smallTextureScale,
            smallTextureScale
          )
        end
      end
    end
  end

  if not self.finalised then
    love.graphics.setColor(1,1,1,0.8)
    local noPoints = #self.points
    local deltaX = self.points[noPoints].x - self.nextPoint.x - grassWidth*0.5
    local deltaY = self.points[noPoints].y - self.nextPoint.y
    local numberOfBlades = math.ceil(math.abs(deltaX/grassWidth))
    for j = 1, numberOfBlades-1 do
      local translatedX = snapTo(self.points[noPoints].x + j*grassWidth*(-deltaX/math.abs(deltaX)), smallTextureScale)
      local translatedY = snapTo(self.points[noPoints].y - (j/(numberOfBlades-1))*deltaY, smallTextureScale)
      
      if grassRNG:random() <= self.flowerCoverage then
        grassTexture = texture.grassFlower[grassRNG:random(1,3)]
      else
        grassTexture = texture.grass[grassRNG:random(1,3)]
      end
      

      blue.graphics.drawCentredShaded(grassTexture,
        translatedX, 
        translatedY,
        0,
        smallTextureScale,
        smallTextureScale)
    end
  end
  if debugMode then
    for i, point in ipairs(self.points) do
      love.graphics.setColor(1,0.1,0.1)
      blue.graphics.circle("line", point.x, point.y, pointRadius)
    end
  end
end


function Grass:delete()
  self.remove = true
end


function Grass:updateBuilder()
  if love.keyboard.isScancodeDown("delete") and builder.mode == "grass" then
    for i, point in ipairs(self.points) do
      local mouseWorldVector = Vec2(mouse.worldX, mouse.worldY)
      if vector.getMagnitudeSqrd(point - mouseWorldVector) <= pointRadius^2 then
        self:delete()
      end
    end
  end
end
  



function Grass.Eval(x, p1, p2)
  local m = (p2.y - p1.y)/(p2.x - p1.x)
  local c = p1.y - m*p1.x
  
  
  return m*x + c
end