Door = Interactable:extend()

function Door:new(x, y, attributes)
  Door.super.new(self, x, y, attributes)
  createAttribute(self.attributes, "doorType", doorTexture or "wooden")
  createAttribute(self.attributes, "doorLocked", false)
  createAttribute(self.attributes, "autoCloseTimer", 10)
  createAttribute(self.attributes, "canClose", true)
  self.type = "door"
  self.isOpen = false
  self.openTimer = 0
  self.openRight = true
  
  --Remove unneeded attributes
  self.attributes.painted = nil
  self.attributes.foreground = nil
  self.attributes.rotation = nil
  self.attributes.animation = nil
  
  if not attributes then
    self.attributes.interactRadius = 75
    self.attributes.requiresFacing = true
  end
  
  
  
  self.body = love.physics.newBody(world, x, y, "static")
  self:setDimentions()
  self.shape = love.physics.newRectangleShape(0, 0, self.width, self.height)
  self.fixture = love.physics.newFixture(self.body, self.shape, 1)
  
end


function Door:setDimentions()
  local doorSet = texture.doors[self.attributes.doorType]
  self.width = (doorSet.closed:getWidth() - 2 - 8)*smallTextureScale
  self.height = (doorSet.closed:getHeight() - 2)*smallTextureScale
end


function Door:activate(playerPosition)
  
  if not self.isOpen then
    self:open(playerPosition)
  else
    self:close()
  end  
end

function Door:isInteractable()

  return self.active ~= false and (not  self.attributes.doorLocked) and (not (self.attributes.canClose == false and self.isOpen))
end



function Door:update(dt)
  
  if self.openTimer ~= 0 then
    if self.attributes.autoCloseTimer > 0 and self.openTimer - dt < 0 then
      self:close()
    else
      self.openTimer = math.max(0, self.openTimer - dt)
    end
  end
  
  if (not self.attributes.canClose) and self.open then
    self.active = false
  elseif (not self.attributes.canClose) then
    self.active = true
  end
  
  
  
end


function Door:open(playerPosition)
  playerPosition = playerPosition or {x = 1}
  self.openRight = playerPosition.x < 0
  self.isOpen = true
  self.fixture:setFilterData(1,0,0)
  self.openTimer = self.attributes.autoCloseTimer
end


function Door:close()
  self.isOpen = false
  self.openTimer = 0
  self.fixture:setFilterData(1, 65535,0)
end


function Door:setlock(input)
  --Toggle lock if no input
  
  self.attributes.doorLocked = not self.attributes.doorLocked
  
  if input == nil then
    self.attributes.doorLocked = input
  end
end




function Door:draw()
  love.graphics.setColor(1, 1, 1)
  

  
  local doorSet = texture.doors[self.attributes.doorType]

  if self.isOpen then
    
    if self.openRight then
      blue.graphics.drawCentredShaded(doorSet.open, self.x - doorSet.ox, self.y, 0, -smallTextureScale, smallTextureScale)
    else
      blue.graphics.drawCentredShaded(doorSet.open, self.x + doorSet.ox, self.y, 0, smallTextureScale, smallTextureScale)
    end
  else
    blue.graphics.drawCentredShaded(doorSet.closed, self.x, self.y, 0, smallTextureScale, smallTextureScale)
  end
  
  
end

function Door:overlayDraw()
  if debugMode then
    love.graphics.setColor(1, 0, 0)
    blue.graphics.polygon("line", {self.body:getWorldPoints(self.shape:getPoints())})
  end

  if builderMode then
    if self.selected and builderMode then
      love.graphics.setColor(1, 1, 1)
      
      local width = self.width
      local height = self.height

      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - width/2, self.y - height/2, width, height)
      love.graphics.setLineWidth("1")
    end
  end


  if debugMode or (self.selected and builderMode) then
    if self.attributes.interactRadius and self:isInteractable() then
      love.graphics.setColor(1,0,0)
      blue.graphics.circle("line", self.x, self.y, self.attributes.interactRadius)
    end
  end

  self:drawInteractPrompt()
end