Interactable = Prop:extend()


function Interactable:new(x, y, attributes)
  Interactable.super.new(self, x, y, attributes)
  self.type = "interactable"
  createAttribute(self.attributes, "priority", false)
  createAttribute(self.attributes, "interactRadius", 100)
  createAttribute(self.attributes, "requiresFacing", true)
  createAttribute(self.attributes, "singleUse", false)
  createAttribute(self.attributes, "triggerTarget", "")

  self.active = true


end


function Interactable:interact(playerPosition)
  -- Player position is relative to the interactable

  if (not self.active) and (not self.attributes.doorLocked) then -- this section should never activate
    cout("Error with interact! An inactive "..self.type.." cannot be interacted with!" )
    return
  end

  if (self.attributes.singleUse and self.active) or (not self.attributes.singleUse) then
    if self.attributes.name ~= "" and specialInteract[self.attributes.name] then
      specialInteract[self.attributes.name](self, playerPosition)
    end
    if self.attributes.singleUse then self.active = false end
    self:activate(playerPosition)
    
    if self.attributes.triggerTarget ~= "" then
      
      for i, target in ipairs(nameIndex.entities[self.attributes.triggerTarget]) do
        if target.activate then
          target:activate()
        end
      end
    end
    
    
  else
    cout("Error with interact! This message should not be able to flash" )
  end
end


function Interactable:update()
  if specialUpdate[self.attributes.name] then
    specialUpdate[self.attributes.name](self)
  end
  self:updateAnimation(dt)
end


--Return if the object can be interacted with. What the game uses to decide what can and can't be interacted with
function Interactable:isInteractable()
  return self.active ~= false
end



function Interactable:activate(playerPosition)
  
  if not (self.attributes.triggerTarget ~= "" and nameIndex.entities[self.attributes.triggerTarget]) then
    cout("WARNING: Interactable has no behaviour!")
  end
end