SlidingDoor = Door:extend()

function SlidingDoor:new(x, y, attributes)
  SlidingDoor.super.new(self, x, y, attributes)
  
  self.type = "slidingDoor"
  
  --self.attributes.doorType = nil
  --createAttribute(self.attributes, "textureName", or attributes.textureName)
  createAttribute(self.attributes, "openSpeed", 50)
  self.openPosition = Vec2(0, -self.height)
  
  --If the door is opening or closing or still
  --  0 - still
  --  1 - opening
  -- -1 - closing
  
  self.moving = 0
end



function SlidingDoor:activate(playerPosition)
  if not self.isOpen then
    self:open(playerPosition)
  else
    self:close()
  end
end


function SlidingDoor:open(playerPosition)
  playerPosition = playerPosition or {x = 1}
  
  self.isOpen = true
  self.moving = 1
  self.openTimer = self.attributes.autoCloseTimer

end


function SlidingDoor:close(playerPosition)
  playerPosition = playerPosition or {x = 1}

  self.isOpen = false
  self.moving = -1
  self.openTimer = 0
  
end



function SlidingDoor:update(dt)
  
  if self.openTimer ~= 0 then
    if self.attributes.autoCloseTimer > 0 and self.openTimer - dt < 0 then
      self:close()
    else
      self.openTimer = math.max(0, self.openTimer - dt)
    end
  end
  
  
  if self.moving ~= 0 then
    local slideVector = vector.normalize(self.openPosition)
    local speed = self.attributes.openSpeed * self.moving
    
    self.body:setPosition(self.body:getX() + slideVector.x*dt*speed, self.body:getY() + slideVector.y*dt*speed)

    local bodyVector = Vec2(self.body:getPosition())
    
    if self.moving == 1 then
      
      if vector.getMagnitudeSqrd(self + self.openPosition - bodyVector) < (speed*dt)^2 then
        self.moving = 0
        self.body:setPosition(self.x + self.openPosition.x, self.y + self.openPosition.y)
      end
    else
      if vector.getMagnitudeSqrd(self - bodyVector)^2 < (speed*dt)^2 then
        self.moving = 0
        self.body:setPosition(self.x, self.y)
      end
    end
    
  
  
    
    
  end
end


function SlidingDoor:draw()
  love.graphics.setColor(1, 1, 1)
  

  
  local doorSet = texture.doors[self.attributes.doorType]
  
  --Since it is the body that moves, the body is where it is drawn
  blue.graphics.drawCentredShaded(doorSet.closed, self.body:getX(), self.body:getY(), 0, smallTextureScale, smallTextureScale)
  
end


function SlidingDoor:overlayDraw()
  if debugMode then
    love.graphics.setColor(1, 0, 0)
    blue.graphics.polygon("line", {self.body:getWorldPoints(self.shape:getPoints())})
    love.graphics.setColor(1, 1, 1)
    blue.graphics.rectangle("line", self.x - self.width/2, self.y - self.height/2, self.width, self.height)

  end

  if builderMode then
    if self.selected and builderMode then
      love.graphics.setColor(1, 1, 1)
      
      local width = self.width
      local height = self.height

      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - width/2, self.y - height/2, width, height)
      love.graphics.setLineWidth("1")
    end
  end


  if debugMode or (self.selected and builderMode) then
    if self.attributes.interactRadius and self:isInteractable() then
      love.graphics.setColor(1,0,0)
      blue.graphics.circle("line", self.x, self.y, self.attributes.interactRadius)
    end
  end

  self:drawInteractPrompt()
end