Player = Entity:extend()

local normalFiction = 2
local mouseRelativeToPlayer

local testVector = Vec2(0,1)

--Class for the player character

function Player:new(x, y)
  Player.super.new(self, x, y)
  self.activeStage = nil --Make the player always active
  table.insert(drawTable, self)
  self.directionVector = Vec2(1, 0) 
  self.type = "player"
  self.character = "Drea"

  self.body = love.physics.newBody(world, x, y, "dynamic")
  self.body:setSleepingAllowed(false)

  
  -- the player's shape has a radius of 20
  self.shape = love.physics.newCircleShape(20-0.65)
  -- Attach fixture to body and give it a density of 1.
  self.fixture = love.physics.newFixture(self.body, self.shape, 1)
  self.fixture:setRestitution(0.3) -- let the player bounce
  self.fixture:setCategory(2)
  self.body:setFixedRotation(true)
  --self.body:setLinearDamping( 0.05 )

  self.flightMode = "walking"
  self.tucked = false
  self.flapCooldown = 8/12
  self.timeOfLastFlap = 0
  self.flapping = false
  self.dragVector = Vec2(0,0)
  self.liftVector = Vec2(0,0)
  self.fixture:setFriction(normalFiction)
  self.correctionRotationVelocity = 0

  self.fixture:setUserData("player")
  self.contactWithGround = false
  self.tangentWithGround = Vec2(-1, 0)
  self.normalWithGround = false
  self.velocityLastFrame = Vec2(0,0)

  

  self.tangentVelocity = Vec2(0,0)
  velocityVector = Vec2(0,0)

  self.collisionTable = {}
  self.collisionVectorTable = {}
  self.bonked = 0
  self.bonkImmunity = 0

  self.stasis = false
  self.stasisCountdown = 0
  self.timeTucked = 0


  self.wetness = 0
  --self.numberOfSwords = 0

  self.textureSet = "drea"
  self.lookDirection = 1
  self.walkRotation = 0
  self.animation = nil

  self.animationInfo = {}
  self.animationInfo.walking = nil
  self.animationInfo.lastHitGround = nil
  self.animationInfo.timeOfLastContact = 0
  self.animationInfo.sleeping = true

  self.anim = {}
  self.anim.running  = AnimationPlayer(animation.drea_running)
  self.anim.flapping = AnimationPlayer(animation.drea_flapping)
  self.anim.gliding  = AnimationPlayer(animation.drea_gliding)

  self.parried = 0
  self.magicParticleGen = ParticleGen(0,0, 10.0, texture.magic_decay)
  self.stickVector = Vec2(0,0)

  self.fire = true
  self.timeBubble = false

  self.timeWarp = 2
end



function Player:setPosition(x, y)

  if type(x) == "table" then
    --If the first argument is a vector, use it
    self.body:setPosition(x.x, x.y)
  else
    self.body:setPosition(x, y)
  end

end


function Player:getVelocityVector()
  
  return Vec2(self.body:getLinearVelocity())
end


function Player:redirect(x, y)
  
  local speed = vector.getMagnitude(Vec2(self.body:getLinearVelocity()))

  if type(x) == "table" then
    --If the first argument is a vector, use it
    
    
    self.body:setLinearVelocity(x.x*speed, x.y*speed)
    self.directionVector = x
  else
    
    self.body:setLinearVelocity(x*speed, y*speed)
    self.directionVector = Vec2(x, y)
  end

end


function Player:hoveringBehaviour()
  if not self.tucked then
    if controls.hover.thisFrame then

      self.flightMode = toggle(self.flightMode, "hovering", "swooping")
      if self.flightMode == "swooping" then
        self.directionVector = vector.normalize(mouseRelativeToPlayer)
        if self.directionVector.x == 0 and self.directionVector.y == 0 then
          self.directionVector = Vec2(1,0)
        end
      end
      self:untuck()
    end
    if self.flightMode == "hovering" then
      self:hoverControls()
    end
  end
end

function Player:tuckingBehaviour(dt)

  if self.bonked > 0 then
    self:tuck(true)
    self.bonked = self.bonked - dt
  end
  self.bonkImmunity = self.bonkImmunity - dt

  if self.tucked and not self.animationInfo.isSqueezing then
    self.directionVector = vector.angleToVector(self.body:getAngle()) 
    if self.contactWithGround and self.bonked <= 0 then
      if controls.walkLeft.isDown then
        --self.body:setAngularVelocity(self.body:getAngularVelocity()+dt*-5)
        self.body:applyTorque(-400)
      end
      if controls.walkRight.isDown then
        --self.body:setAngularVelocity(self.body:getAngularVelocity()+dt*5)
        self.body:applyTorque(400)
      end
    end
    
    if controls.flap.thisFrame and self.bonked <= 0 then
     self:rollJump(dt)
    end
  end


  if controls.tuck.thisFrame or self.bonked > 0 then
    self:tuck()
  elseif controls.tuck.upThisFrame or (self.bonked <= 0 and not controls.tuck.isDown ) then
    self:untuck()
  end 

  --Setting angular velocity instead of fixing rotation makes it so that if you bonk you will roll properly
  --[[if not self.tucked then
    cout(self.body:getAngularVelocity())
    self.body:setAngularVelocity(self.body:getAngularVelocity())
    --self.body:setAngularVelocity(0) 
  end]]
end



local lastjoystickX = 0
local lastjoystickY = 0

local function getFlightControlDirection()
  
  if gamepadMode then
    
    local safezoneRadius = 0.5
    
    if vector.getMagnitudeSqrd(flightJoystick) > safezoneRadius^2 then
      local direction = vector.normalize(flightJoystick)
      lastjoystickX = direction.x
      lastjoystickY = direction.y
      
      return direction
    else
      
      return Vec2(lastjoystickX, lastjoystickY)
    end
  else
    return vector.normalize(mouseRelativeToPlayer)
  end
end



function Player:flyingBehaviour(dt)
  
  self.timeOfLastFlap = self.timeOfLastFlap - dt
  
  self.isFlapCooldown = self.timeOfLastFlap >= 0
  
  if not self.animationInfo.isSqueezing then
    if controls.flap.isDown then
      if not self.tucked then
        --TODO - Contain all the cooldown stuff in flap(), maybe also add a cooldown override argument
        if not self.isFlapCooldown then
          --Set flap timer to current time
          if self.flightMode == "hovering" or self.flightMode == "walking" then
            self.flightMode = "swooping"
            self.directionVector = getFlightControlDirection()
            if self.directionVector.x == 0 and self.directionVector.y == 0 then
              self.directionVector = Vec2(1,0)
            end
          end
          self:flap()
        end
      end
    end
  
  -- Smooth rotation ==================
    local rotationSpeed = math.max(540 / (vector.getMagnitude(velocityVector)*0.002+1), 120)
    

    if self.flightMode == "swooping" and not self.tucked and not controls.lockDirection.isDown then
      
      local controlDirection = getFlightControlDirection()
      
      
      local mouseAngle = vector.getOrientation(controlDirection)
      local directionAngle = vector.getOrientation(self.directionVector)
      --Degrees/second
      local rotationStep = math.rad(rotationSpeed)*dt
    
      --If the difference in angle is less than a rotation step, just set angle to the mouse
      if math.abs(mouseAngle - directionAngle) <= rotationStep then
        self.directionVector = vector.normalize(controlDirection)


      elseif vector.angleIsClockwise(self.directionVector, controlDirection) then      
        --Clockwise
        self.directionVector = vector.angleToVector(directionAngle + rotationStep) 
      else 
        --Anticlockwise 
        self.directionVector = vector.angleToVector(directionAngle - rotationStep) 
      end
    end

    -- Aerodynamics area for swooping mode ============================================
    if self.tucked == false and self.flightMode == "swooping" then
      local normal = vector.normalize(Vec2(self.directionVector.y, self.directionVector.x * -1))
      local velocityNormal = vector.normalize(Vec2(velocityVector.y, velocityVector.x * -1))
      --Profile of the line from the velocity vector's position
      local areaVector = vector.project(self.directionVector, velocityNormal)
      --The normal, but with the direction the right direction
      local correctedNormal = vector.normalize(vector.project(velocityVector, normal))
      --Final drag vector
      self.dragVector = correctedNormal * -0.004 * vector.getMagnitude(areaVector) * vector.getMagnitudeSqrd(velocityVector)
      self.body:applyForce(self.dragVector.x, self.dragVector.y)
      
      
      --Apply linear drag
        
      local minDragSpeed = 30*love.physics.getMeter()
      if vector.getMagnitudeSqrd(velocityVector) >= minDragSpeed^2 then
        local dragFactor = 0.2
        self.body:applyForce(-velocityVector.x*dragFactor, -velocityVector.y*dragFactor)
      end
      
      
    elseif self.tucked == false and self.flightMode == "hovering" then
      self:walkingBehaviour()
    end

    --If moving slow enough, don't bounce
    if vector.getMagnitudeSqrd(velocityVector) < (10*64)^2 and not self.tucked then
      self.fixture:setRestitution(0)
    else
      self.fixture:setRestitution(0.3)
    end


    --Flap wings if valid

    local flapRatio = 0.3
    
    if ((1-flapRatio)*self.flapCooldown <= self.timeOfLastFlap) and self.tucked == false then
      
      
      --Flap harder if moving slowly
      local maxBonusForce = 300
      local speedSqrd = vector.getMagnitudeSqrd(velocityVector)
      
      local forceBonus = 1/(speedSqrd*0.0000001 + (1/maxBonusForce))
      
      local forceMult = 140 + forceBonus
      tester = forceBonus
      
      local force = forceMult/flapRatio
      
      --Just to help get unstuck
      if self:isStuck() then
        force = force * 10
      end
      
      
      self.body:applyForce(self.directionVector.x * force, self.directionVector.y * force)
    else
      self.flapping = false
    end

    if self.tucked == false and self:canWalk() and not self.flapping then
      self.flightMode = "walking"
    elseif self.flightMode == "swooping" and math.deg(vector.angleBetween(self.directionVector, velocityVector)) > 120 and not self.flapping and vector.getMagnitude(velocityVector)/64 > 3 and (not self.tucked) and not self.stasis  then
      if controls.flap.isDown then

        local vx, vy = self.body:getLinearVelocity()

        self.body:applyForce(-vx, -vy)
      else
        --Stall if moving too fast backwards
        self.flightMode = "walking"
      end
    end
  end
end


function Player:airWalkingBehaviour()
  if not self:canWalk() then
    local maxForce = 25

    if controls.walkRight.isDown and controls.walkLeft.isDown then
      --Do nothing
    elseif controls.walkRight.isDown then
      self.body:applyForce(maxForce, 0)

    elseif controls.walkLeft.isDown then
      self.body:applyForce(-maxForce, 0)
    end
  end
end


function Player:timeBubbleBehaviour()


  if controls.timeBubble.thisFrame then
    if self.timeBubble then
      self.timeBubble = false
    else
      self.timeBubble = 150
      shader.timeBubble:send("bubbleRadius", self.timeBubble)
    end
  end
end

local parryTime = 2
function Player:stickyParryBehaviour(dt)


  self.parried = self.parried - dt
  local parryWindow = 0.2

  if self.tucked and (self.parried <= 0) and love.timer.getTime() - self.timeTucked <= parryWindow and next(self.collisionTable) ~= nil and self.bonked <= 0 and self.collisionImpulse >= 200 then
    local sound = sound.effect.bounce_parry:clone()
    sound:play()
    self.parried = parryTime

  else

    if (self.stickAngleWithLast and math.deg(self.stickAngleWithLast) >= 90) or vector.getMagnitude(vector.subtraction(self.stickVector, Vec2(self.body:getPosition()))) > vector.getMagnitude(Vec2(self.body:getLinearVelocity()))*3*dt then
      self.parried = 0
    end
  end






  if self.parried > 0 and controls.tuck.isDown then

    local vx, vy = self.body:getLinearVelocity()

    local velocityVector
    local speed 
    if parryTime == self.parried then
      --If this is the frame that it was parried, use the velocity last frame to prevent slowing down due to bouncing
      velocityVector = self.velocityLastFrame
      speed = vector.getMagnitude(velocityVector)*0.8
    else
      velocityVector = Vec2(vx, vy)
      speed = vector.getMagnitude(velocityVector)
    end


    local newVelocityVector = vector.normalize(vector.project(velocityVector, self.stickyDirection))* speed

    self.body:setLinearVelocity(newVelocityVector.x, newVelocityVector.y)

    self.body:setPosition(self.stickVector.x + vx*dt, self.stickVector.y + vy*dt)
    
    
    
  end


  if controls.tuck.isDown then
    self:getStickyVector()
  end

end


function Player:blowbackBehaviour(dt)
  

  if self.flightAssist then
  
    if self.flightMode == "swooping" and not self.tucked then
      
      local minimumSpeed = 18 * love.physics.getMeter()
      local minimumDistance = 500
      
      local atSpeed = minimumSpeed^2 <= vector.getMagnitudeSqrd(velocityVector)
      --local withinDistance = minimumDistance^2 >= vector.getMagnitudeSqrd(self.blowbackVector)
      
      
      if atSpeed then
        
        local rayLength = minimumDistance
        

        --I'm going to sort by distance by only looking at one axis, since it's on a 
        --straight line the distance can be compared with only one value with no need
        --for pythagoras, as long as the axis changes (which will be determined at the 
        --start). This might be overoptimising.
        
        local xSortMode
        
        local castStartX = self.body:getX()
        local castStartY = self.body:getY()
        local castEndX   = castStartX + self.directionVector.x * rayLength
        local castEndY   = castStartY + self.directionVector.y * rayLength
        
        local closestHit
        local closestHitNormal
        local closestDistance 
        
        local function setClosestHit(x, y, xn, yn)
          closestHit       = Vec2(x, y)
          closestHitNormal = Vec2(xn, yn)
          if xSortMode then
            closestDistance = math.abs(castStartX - x)
          else
            closestDistance = math.abs(castStartY - y)
          end
        end
        
        
        
        
        local function callback(fixture, x, y, xn, yn, fraction)
            --Set the sort mode and first values
            
          local userData = fixture:getUserData()
          
          --Exclude fixtures that can't be collided with
          if fixture:getCategory() == 2 then
            return 1
          end

          

          if xSortMode == nil then
          
            xSortMode = math.abs(castStartX - x) > math.abs(castStartY - y)
            setClosestHit(x, y, xn, yn)
            
          else
            
            if xSortMode then
              if closestDistance > math.abs(castStartX - x) then
                setClosestHit(x, y, xn, yn)
              end
            else
              if closestDistance > math.abs(castStartY - y) then
                setClosestHit(x, y, xn, yn)
              end
            end
            
          end
          return 1
        end
        
        world:rayCast(castStartX, castStartY, castEndX, castEndY, callback)
        
        --Closest point aquired
        self.closestPoint       = closestHit
        self.closestPointNormal = closestHitNormal
        
        local allignmentThreshold = 0
        
        if self.closestPoint then
          local allignment = math.max(vector.dot(-self.closestPointNormal, self.directionVector), 0.2)
          
          
          if allignment > allignmentThreshold then
            
            local distanceMult = minimumDistance/(vector.getMagnitude(Vec2(self.body:getPosition()) - self.closestPoint))-0.5
            
            local rotationSpeed = 2400 * allignment * distanceMult
            
            
            
            local rotationStep = math.rad(rotationSpeed)*dt
            local directionAngle = vector.getOrientation(self.directionVector)
            
            
            if vector.angleIsClockwise(-self.closestPointNormal, self.directionVector) then      
              --Clockwise

              self.directionVector = vector.angleToVector(directionAngle + rotationStep) 
            else 
              --Anticlockwise 

              self.directionVector = vector.angleToVector(directionAngle - rotationStep) 
            end
            
            
            --Going to use momentum based rotation
            
           --[[ if vector.angleIsClockwise(-self.closestPointNormal, self.directionVector) then      
              --Clockwise
              self.correctionRotationVelocity = math.max(math.abs(self.correctionRotationVelocity),  1200*allignment)
            else 
              --Anticlockwise 
              self.correctionRotationVelocity = -math.max(math.abs(self.correctionRotationVelocity),  1200*allignment)
            end]]
            

          end
          
        end
        
      end
    end
  end
end


function Player:fireballBehaviour()
  
  if self.fire and controls.fireball.thisFrame and (not self.tucked) and (not self.animationInfo.isSqueezing) then
    local magnitude = 700

    local vx, vy = self.body:getLinearVelocity()

    local offset = self.directionVector * self.shape:getRadius() +  Vec2(0, -5)
  
    local target = mouseRelativeToPlayer - offset
    
    if gamepadMode then
      target = vector.normalize(Vec2(joysticks[1]:getGamepadAxis("rightx"), joysticks[1]:getGamepadAxis("righty")))
    else
      target = mouseRelativeToPlayer - offset
    end
    
    
    local launchPosition = Vec2(self.body:getPosition()) + offset


    local direction = vector.normalize(target)
    --Direction vector does not work properly as an offset
    Fireball(
      launchPosition.x, 
      launchPosition.y, 
      direction.x*magnitude + vx,
      direction.y*magnitude + vy)
    self.fire = false
    self.animationInfo.sleeping = false
  end
end


function Player:stopSqueezing()
  self.sqeezeTrack = nil
  self.body:setActive(true)
  self.animationInfo.isSqueezing = nil
end


function Player:squeezeBehaviour(dt)
  
  self.wetness = self.wetness - dt
  
  local squeezeTrack = self.sqeezeTrack
  
  if squeezeTrack then
    
    local newPos, dir = squeezeTrack:getPositionFromDistance(self.squeezeDistance + self.squeezeSpeed*dt)
    
    
    
    
    self.squeezeDistance = self.squeezeDistance + self.squeezeSpeed*dt
    
    if newPos then
      if self.squeezeSpeed < 0 then
        dir = dir *-1
      end
        
      self.directionVector = dir
      local vel = dir * math.abs(self.squeezeSpeed)
      self.body:setActive(false)
      self.body:setPosition(newPos.x, newPos.y)
      self.body:setLinearVelocity(vel.x, vel.y)
      
    else
      self:stopSqueezing()
    end
    
  end
  
end
  


function Player:isStuck()
  
  local normals = self.collisionTable
  
  return normals[2] and vector.angleBetween(normals[1], normals[2]) >= math.rad(90)

  
end



function Player:interactBehaviour()

  if self.tucked == false and self.flightMode == "walking" then
    local closestInteractable = self:checkInteractables()
    if closestInteractable then
      closestInteractable.nearest = true
    end

    if controls.interact.thisFrame then
      self:interact(closestInteractable)
    end
  end
end




function Player:getStickyVector()
  --Now need to find the closest point

  local closestDistanceSqrd
  local closestPoint
  local stickyDirection


  for i, polygon in ipairs(polygonTable) do

    if polygon.stickTracks then
      for i = 1, #polygon.stickTracks, 2 do

        local x1 = polygon.stickTracks[i]
        local y1 = polygon.stickTracks[i+1]

        local x2 = polygon.stickTracks[(i+1) % #polygon.stickTracks + 1]
        local y2 = polygon.stickTracks[(i+2) % #polygon.stickTracks + 1]


        local point = line.getClosestPoint(x1, y1, x2, y2, self.body:getPosition())
        local displacementVector = vector.subtraction(point, Vec2(self.body:getPosition()))
        local distanceSqrd = vector.getMagnitudeSqrd(displacementVector)

        if (not closestDistanceSqrd) or distanceSqrd < closestDistanceSqrd then
          closestPoint = point
          closestDistanceSqrd = distanceSqrd

          stickyDirection = Vec2(x2 - x1, y2 - y1)
        end
      end
    end
  end
  if self.stickyDirection then
    self.stickAngleWithLast = vector.angleBetween(self.stickyDirection, stickyDirection)
  end
  self.stickyDirection = stickyDirection
  self.stickVector = closestPoint

end



function Player:cameraZoomBehaviour()
  local foundPolygon = false
  if self.flightMode == "swooping" then
    camera.maxZoom = 1
  else
    for i, polygon in ipairs(polygonTable) do
      if polygon.attributes.zoomCamera > 0 then
        if polygon:checkInside(self.body:getPosition()) then
          camera.maxZoom = polygon.attributes.zoomCamera
          foundPolygon = true
          break
        end
      end
    end
    if not foundPolygon then
      camera.maxZoom = 2
    end
  end
end


function Player:flap()
  

  sound.effect.wingBeat:setVolume(1)
  sound.effect.wingBeat:play()
  self.animationInfo.sleeping = false
  self.animationInfo.walking = false
  self.flightMode = "swooping"
  --TODO - change flapping to use dt instead of time
  self.timeOfLastFlap = self.flapCooldown
  self.flapping = true
end




function Player:rollJump(dt, forceJump)

  if self.contactWithGround or self.parried > 0 or love.timer.getTime() - self.animationInfo.timeOfLastContact <= 0.15 or forceJump then
    sound.effect.wingBeat:play()

    -- Set bonk immunity so we don't bump on the ground
    self.bonkImmunity = 0.1
    self.parried = 0
    
    
    
    local contactNormal = Vec2(0,0)
    
    for i, normal in pairs(self.collisionTable) do
      contactNormal = contactNormal + normal
    end
    contactNormal = vector.normalize(contactNormal)
    
    if contactNormal.x == 0 and contactNormal.y == 0 then
      contactNormal = self.lastContactNormal
    else
      self.lastContactNormal = contactNormal
    end
    
    local direction
    local controlDirection = getFlightControlDirection()
    if contactNormal then
      if vector.dotProduct(contactNormal, controlDirection) > 0 then
        direction = controlDirection
      else
        
        local mirrorLine = vector.turn90Degrees(contactNormal)
        direction = vector.normalize((controlDirection - 2 * vector.dotProduct(controlDirection, mirrorLine) * mirrorLine)*-1)
        
      end
      
      

      --Redirect the previous velocity
      local boostAmount = 200
      local speed = vector.getMagnitude(velocityVector)*0.70 + boostAmount
      local newVelocity = direction*speed
      
      self.flightMode = "walking"
      
      self.body:setLinearVelocity(newVelocity.x, newVelocity.y)
      
      TempAnim(self.body:getX(), self.body:getY(), vector.getOrientation(direction)+math.rad(90), animation.effect_magicRipple)



    end
  end
end



function Player:tuck(noRotationBoost)
  
  self.animationInfo.sleeping = false
  if not self.tucked and not self.stasis then
    self.timeTucked = love.timer.getTime()
    --self.fixture:setFriction(0.2)
    self.tucked = true
    self.body:setAngle(vector.getOrientation(self.directionVector))



    if not noRotationBoost then
      local newRotationSpeed = self.body:getLinearVelocity()/self.shape:getRadius() * 1.2

      if controls.walkLeft.isDown and not controls.walkRight.isDown then
        newRotationSpeed = -math.min(10, math.abs(newRotationSpeed))
      elseif controls.walkRight.isDown then
        newRotationSpeed = math.min(10, math.abs(newRotationSpeed))
      else
        newRotationSpeed = math.max(math.min(10, newRotationSpeed), -10)
      end

      self.animationInfo.walking = false

      self.body:setFixedRotation(false)
      self.body:setAngularVelocity(newRotationSpeed)
    end
  end
end

function Player:untuck()
  
  if self.tucked then
    self.fixture:setFriction(normalFiction)
    self.tucked = false
    
    if self.animationInfo.isSqueezing then
      if self.flightMode == "swooping" then
        self.directionVector = vector.normalize(getFlightControlDirection())
      else
        self.directionVector = Vec2(self.lookDirection,0)
      end
    end
    self.parried = 0
    
    self.body:setFixedRotation(true)
    
    --self.body:setAngularVelocity(0)
  end
end

function Player:hoverControls()
  self.dragVector = vector.normalize(velocityVector) * -0.4*vector.getMagnitude(velocityVector)
  if not love.keyboard.isScancodeDown("down", "s") then
    self.body:applyForce(0, -gravity * self.body:getMass())
  end
  self.body:applyForce(self.dragVector.x, self.dragVector.y)

  local hoverVector = Vec(0)

  if controls.walkRight.isDown then
    hoverVector = hoverVector + Vec2(1, 0)
  end
  if controls.walkLeft.isDown then
    hoverVector = hoverVector + Vec2(-1, 0)
  end
  if love.keyboard.isScancodeDown("up", "w") and not love.keyboard.isScancodeDown("down", "s") then
    hoverVector = hoverVector + Vec2(0, -1)
  end

  hoverVector = vector.normalize(hoverVector)
  local maxForce = 120 + gravity * self.body:getMass()
  local wingForce = hoverFlightFunction(Vec2(0, -gravity * self.body:getMass()), hoverVector, maxForce)
  hoverVector = hoverVector * wingForce

  self.body:applyForce(hoverVector.x, hoverVector.y)
  self.directionVector = vector.normalize(hoverVector + Vec2(0, -gravity*0.5 * self.body:getMass()))
end


function Player:canWalk()
  local maxAngle = math.rad(60)
  if self.contactWithGround then
    return math.acos(vector.dotProduct(self.normalWithGround, Vec2(0,-1))) <= maxAngle
  else 
    return false
  end
end


function Player:canWalkPart(normal)
  local maxAngle = math.rad(60)
  return math.acos(vector.dotProduct(normal, Vec2(0,-1))) <= maxAngle
end



function Player:walkingBehaviour()
  if self.stasisCountdown <= 0 and self:canWalk() and not self.animationInfo.isSqueezing then
    --Need to get relative tangential velocity without losing the sign
    --Project on to x axis, then normalize. Then multiply the resulting vector by the magnitude 
    
    self.falling = false
    
    local horizontalTangentVector = vector.project(self.tangentVelocity, Vec2(1,0))
    horizontalTangentVector = horizontalTangentVector * vector.getMagnitude(self.tangentVelocity)
    local relativeGroundVelocity = vector.project(velocityVector, self.tangentVelocity) * 0.00002
    local contactWithGround = ((180 - math.deg(vector.angleBetween(self.normalWithGround, Vec2(0,-1)))*2))/180
    --Fraction based on the slope the ground is on
    

    local directionForce = vector.normalize(self.tangentWithGround) -- Vector
    
    
    if  (controls.walkLeft.isDown and controls.walkRight.isDown) or 
    not (controls.walkLeft.isDown or  controls.walkRight.isDown) then
      --If pressing both left and right, or pressing neither
      self.animationInfo.walking = false

      local groundVelocitySqrd = vector.getMagnitudeSqrd(relativeGroundVelocity)

      if groundVelocitySqrd >= 1100^2 then
        self.animationInfo.sliding = true
      elseif groundVelocitySqrd <= 10^2 then
        self.animationInfo.sliding = false
      end


      if self.animationInfo.sliding and vector.getMagnitudeSqrd(self.tangentVelocity) > 0.1^2 then
        self.directionVector = vector.normalize(self.tangentVelocity) 
      end
    else

      self.animationInfo.sleeping = false
      self.animationInfo.walking = true
      self.animationInfo.sliding = false


      if controls.walkLeft.isDown then
        
        self.lookDirection = -1
        local walkForce = 1000 * contactWithGround
        
        
        --directionForce should already be normalized here
        self.directionVector = directionForce
        
        directionForce = (directionForce * walkForce) - relativeGroundVelocity

        if directionForce.x < -walkForce or directionForce.x > walkForce then
          directionForce = vector.normalize(directionForce) * walkForce
        end
      elseif controls.walkRight.isDown then

        self.lookDirection = 1
        local walkForce = - 1000 * contactWithGround

        --directionForce should already be normalized here
        self.directionVector = -directionForce
        
        directionForce = (directionForce*walkForce) - relativeGroundVelocity
        
        --TODO work out what's going on here
        if directionForce.x < -walkForce or directionForce.x > walkForce then
          directionForce = vector.normalize(directionForce) * -walkForce
        end
      end
    end


    self.body:applyForce(directionForce.x, directionForce.y)
  elseif love.timer.getTime() - self.animationInfo.timeOfLastContact > 0.2 then
    self.animationInfo.walking = false
  end
end



local function hoverFlight(g, F, M)
  -- I'm gonna hate myself later for this
  -- Given known vector G, known unit vector F and a known magnitude of the resultant force M.
  -- vector M (which is unknown) must be at full length, and rests on the end of g, and the other on vector tF. tF (t being the magnitude of F) is of unknown length, but the direction is known.

  -- Using pythagorous and the quadratic equation, we can find t

  --      If you're not me in the future and reading this, sorry - I did most of it on paper

  local a = g.x
  local b = g.y
  local c = F.x
  local d = F.y

  local m = a*c + b*d
  local n = c^2 + d^2

  --Time for the quadratic equation :_)

  if 2*n == 0 then
    return 0
  end
  solution = (-(2*m) + math.sqrt((2*m)^2 - 4*n*(a^2 + b^2 - M^2))) / (2*n)

  if solution < 0 then
    print("switched sign")
    solution = (-(2*m) - math.sqrt((2*m)^2 - 4*n*(a^2 + b^2 - M^2))) / (2*n)
  end
  return solution
  --IT WORKS AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
end



local function getDistanceVector(object, playerPos)
  
  if object.type == "squeeze" then
    
    local displacement1 = object.points[1] - playerPos
    local displacement2 = object.points[#object.points] - playerPos
    
    local distance1 = vector.getMagnitudeSqrd(displacement1) 
    local distance2 = vector.getMagnitudeSqrd(displacement2) 
    
    if distance1 < distance2 then
      
      object.x = object.points[1].x
      object.y = object.points[1].y
      
      return displacement1, distance1
    else
      
      object.x = object.points[#object.points].x
      object.y = object.points[#object.points].y
      
      return displacement2, distance2
    end
    
  else
    return object - playerPos, vector.getMagnitudeSqrd(object - playerPos)
  end
  
end



function Player:checkInteractables()
  --Finally, inner peace
  local playerPos = Vec2(self.body:getPosition())
  local closestDistance = nil
  local closestObject = nil
  local priority = false
  
  for i, object in ipairs(drawTable) do
    --Note to self - might be better to only pass a table of interactables
    --Further note to self - this might also be an opitunity to implement quadtrees
    if object.isInteractable and object:isInteractable() then
      --Priority objects will always get through
      if (not priority) or object.attributes.priority then
        --Check if the player is facing the object if it needs to
        
        local distanceVector, newDistance = getDistanceVector(object, playerPos)
        if object.attributes.requiresFacing == false or self.directionVector.x * (object.x - playerPos.x) > 0 then

          
          --Check if the object is within the interact radius
          if newDistance < object.attributes.interactRadius^2 then
            
            --Set default values if no object has been found yet
            closestDistance = closestDistance or newDistance
            closestObject   = closestObject or object
            
            --Check if closer than the closest so far. If not, check if it is a priority object, and if priority mode has not already been set, select this as the closest object
            if newDistance < closestDistance or (object.attributes.priority and not priority) then
              priority        = priority or object.attributes.priority
              closestDistance = newDistance
              closestObject   = object
            end
          end
        end
      end
    end
  end
  
  return closestObject
end


function Player:interact(closestObject)
  local playerPos = Vec2(self.body:getPosition())
  
  if closestObject then
    
    local relativePlayerPosition = vector.subtraction(playerPos, closestObject)
    closestObject:interact(relativePlayerPosition)
  end
end


function Player:keepInBounds()
  local x = self.body:getX() 
  local y = self.body:getY() 

  if x > 60000 then
    self.body:setX(-30000)
  end

  if x < -60000 then
    self.body:setX(30000)
  end



  if y > 15000 then
    self.body:setPosition(0, -10000)
  end

end


function Player:update(dt)
  
  self.fire = true
  velocityVector = Vec2(self.body:getLinearVelocity()) 
  mouseRelativeToPlayer = Vec2(mouse.worldX - self.body:getX(), mouse.worldY - self.body:getY())


  self:getCollisionVectors()

  if self.character == "Drea" or true then
    
    self:squeezeBehaviour(dt)
    self:tuckingBehaviour(dt)
    self:flyingBehaviour(dt)
    self:blowbackBehaviour(dt)
    --self:stasisBehaviour(dt)
    --self:boostBehaviour(dt)
    --self:bounceParryBehaviour()
    self:stickyParryBehaviour(dt)
    self:timeBubbleBehaviour()
    self:fireballBehaviour()
    self:cameraZoomBehaviour()

    if self:isStuck() then
      self.fixture:setFriction(0)
    else
      self.fixture:setFriction(normalFiction)
    end
    

    self:keepInBounds()

    self.falling = (not self.tucked) and self.flightMode == "walking" and love.timer.getTime() - self.animationInfo.timeOfLastContact > 0.3

    if self.falling then

      vx, vy = self.body:getLinearVelocity()


      if not (vx == 0 and vy == 0) and not self.animationInfo.isSqueezing then
        self.directionVector = vector.normalize(Vec2(self.body:getLinearVelocity()))
      end
    end
  end



  if self.tucked == false and self.flightMode == "walking" then
    self:walkingBehaviour()
  end



  

  if self.character == "Sift" then
    --[[if clickedThisFrame.primary and self.numberOfSwords < 2 then
      --Note - fix crash when mouse is at 0,0
      self.numberOfSwords = self.numberOfSwords + 1
      local distanceFromPlayer = 50
      local swordPosition = vector.normalize(mouseRelativeToPlayer) * (20 + distanceFromPlayer)
      local newSword = Sword(swordPosition.x + self.body:getX(), swordPosition.y + self.body:getY(), mouseRelativeToPlayer)
      
      
      if self.numberOfSwords == 1 then
        self.rope = Rope(self, newSword, 1500)
      elseif self.numberOfSwords == 2 then
        newSword.rope = self.rope
        self.rope:setLength(self.rope.defaultLength)
        self.rope:swapBodies(self.body, newSword.body)
      end
      
    end
    if love.mouse.isDown(2) then
      self.numberOfSwords = 0
    end
    
    self:airWalkingBehaviour()
    ]]
  end 

  if controls.lockDirection.thisFrame then
    mouse.storedX = love.mouse.getX()
    mouse.storedY = love.mouse.getY()
  end
  if controls.lockDirection.isDown then
    mouse.storedX = love.mouse.getX()
    mouse.storedY = love.mouse.getY()
  end
  if controls.lockDirection.upThisFrame then
    love.mouse.setPosition(mouse.storedX, mouse.storedY)
  end


  self:interactBehaviour()


  if self.contactWithGround then
    self.animationInfo.timeOfLastContact = love.timer.getTime()
  end


  self.magicParticleGen.rotation = vector.getOrientation(velocityVector) + math.rad(90)
  self.magicParticleGen.x = self.body:getX()
  self.magicParticleGen.y = self.body:getY()
  local minMagic = 0.5
  local maxMagic = 3
  self.magicParticleGen.amount = math.floor(vector.getMagnitudeSqrd(velocityVector)/(160)^2) * math.min(math.max(self.parried*2, 0), maxMagic)/3




  for i, animation in pairs(self.anim) do
    animation:update(dt)
  end


  self.velocityLastFrame = velocityVector

end



function Player:draw()
  blue.graphics.setZ(0)
  
  --Beginning of texture drawing
  love.graphics.setShader(shader.energy)

  local rot = vector.getOrientation(self.directionVector) + math.rad(90)

  if not (self.tangentWithGround.x == 0 and self.tangentWithGround.y == 0) then
    self.walkRotation = vector.getOrientation(self.tangentWithGround) + math.rad(-90)
  else
    if (self.directionVector.x) < 0 then
      self.walkRotation = rot + math.rad(180)
    else
      self.walkRotation = rot
    end
  end
  

  
  local dryness = 1 - math.min(self.wetness*0.25, 0.5)
  love.graphics.setColor(dryness, dryness, 1)


  if (self.flightMode ~= "walking" or self.falling) and not self.tucked then
    if self.directionVector.x > 0 then
      self.lookDirection = 1
    else 
      self.lookDirection = -1
    end
  end

  -- Only one animation at a time! self.animation refers to full animations here, or static images

  if self.animationInfo.sleeping then
    self.animation = "sleeping"
  elseif self.animationInfo.isSqueezing then
    self.animation = "squeezing"
  elseif self.animationInfo.walking then
    self.animation = "running"
  elseif self.tucked then
    self.animation = "tucked"
  elseif self.falling then
    self.animation = "falling"
  elseif self.flightMode == "swooping" and not self.tucked and (self.flapping or self.isFlapCooldown) then
    self.animation = "flapping"
  elseif self.flightMode == "swooping" then
    self.animation = "gliding"
  elseif self.animationInfo.sliding then
    self.animation = "sliding"
  else
    -- If nothing else is detected, then set to idle
    self.animation = "idle"
  end


  do 
    local sts = smallTextureScale
    local x, y = self.body:getPosition()

    if self.animation == "idle" then
      local texture = texture.drea_idle
      blue.graphics.drawCentredShaded(texture, x, y, self.walkRotation, sts*self.lookDirection, sts, texture.ox, texture.oy)
    elseif self.animation == "running" then
      self.anim.running:play(x, y, self.walkRotation, sts*self.lookDirection, sts)
    elseif self.animation == "tucked" then
      local texture = texture.drea_tucked
      blue.graphics.drawCentredShaded(texture, x, y, rot, sts*self.lookDirection, sts, texture.ox, texture.oy)
    elseif self.animation == "sliding" then
      local texture = texture.drea_sliding
      blue.graphics.drawCentredShaded(texture, x, y, self.walkRotation, sts*self.lookDirection, sts, texture.ox, texture.oy)
    elseif self.animation == "falling" then
      local texture = texture.drea_falling
      blue.graphics.drawCentredShaded(texture, x, y, rot, sts, sts*self.lookDirection, texture.ox, texture.oy)
    elseif self.animation == "gliding" or self.animation == "squeezing" then
      self.anim.gliding:play(x, y, rot, sts, sts*self.lookDirection)
    elseif self.animation == "flapping" then
      self.anim.flapping:play(x, y, rot, sts, sts*self.lookDirection)
    elseif self.animation == "sleeping" then
      local texture = texture.drea_sleeping
      blue.graphics.drawCentredShaded(texture, x, y, 0, sts, sts, texture.ox, texture.oy)
    end
  end
  love.graphics.setShader()

  for i, animation in pairs(self.anim) do
    if i ~= self.animation then
      animation:stop()
    end
  end
  
  if self.parried > 0 then
    local x = (parryTime - self.parried)/parryTime
    local intensity = (-x*x + 1) * 2
    addLight(self.body:getX(), self.body:getY(), -40, intensity, {1, 211/255, 162/255})
  end
  
end


function Player:drawOverlay()
  
  if debugMode then 
    if self:isStuck() then
      love.graphics.setColor(0,0,1, 0.5)
    else
      love.graphics.setColor(1,0,0, 0.5)
    end
    blue.graphics.circle("fill", self.body:getX(), self.body:getY(), self.shape:getRadius())
    love.graphics.setColor(0, 0, 0)
    blue.graphics.circle("line", self.body:getX(), self.body:getY(), self.shape:getRadius())


    if self.flightMode == "swooping" and self.tucked == false then
      love.graphics.setColor(0, 0.5, 1)
      blue.graphics.line(self.body:getX() + self.directionVector.x*-80, self.body:getY() + self.directionVector.y*-80, self.body:getX() + self.directionVector.x*80, self.body:getY() + self.directionVector.y*80)
    end


    if self.tucked then
      love.graphics.setColor(0,0,0,0.5)
      blue.graphics.circle("line", self.body:getX(), self.body:getY(), self.shape:getRadius()+5)
    end


    love.graphics.setColor(0, 0, 0)
    blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.directionVector, 35)
    
    
    
    for i, normal in pairs(self.collisionTable) do
      love.graphics.setColor(0, 0.9, 0.5)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), normal, 60)
    end


    if self.normalWithGround and self.contactWithGround then
      love.graphics.setColor(1, 0.9, 0.5)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.normalWithGround, 60)
    end

    if self.stickyVector then
      love.graphics.setColor(1, 0, 0, 0.2)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.stickyVector, 60)
    end
    
    if self.blowbackVector and controls.blowbackMode.isDown then
      love.graphics.setColor(1, 1, 1)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), vector.normalize(self.blowbackVector), 35)
    end
    
    if self.closestPoint then

      love.graphics.setColor(0, 1, 1)
      blue.graphics.circle("line", self.closestPoint.x, self.closestPoint.y, 5)
      
      love.graphics.setColor(0, 0, 1)
      blue.graphics.vectorLine(self.closestPoint.x, self.closestPoint.y, self.closestPointNormal, 50)
      love.graphics.setColor(1, 0, 0)
      blue.graphics.vectorLine(self.closestPoint.x, self.closestPoint.y, self.closestPointNormal, -50)
      
    end
    
    do
    
      local contactNormal = Vec2(0,0)
      
      for i, normal in pairs(self.collisionTable) do
        contactNormal = contactNormal + normal
      end
      contactNormal = vector.normalize(contactNormal)
      
      if contactNormal.x == 0 and contactNormal.y == 0 then
        contactNormal = self.lastContactNormal
      else
        self.lastContactNormal = contactNormal
      end
      
      if contactNormal then
        local direction
        if vector.dotProduct(contactNormal, mouseRelativeToPlayer) > 0 then
          direction = mouseRelativeToPlayer
        else
          local mirrorLine = vector.turn90Degrees(contactNormal)
          direction = (mouseRelativeToPlayer - 2 * vector.dotProduct(mouseRelativeToPlayer, mirrorLine) * mirrorLine)*-1
          
        end
        love.graphics.setColor(1,0,0)
        blue.graphics.vectorLine(self.body:getX(), self.body:getY(), direction)
      end
    end
    
  end
  
end