Prop = Entity:extend()


function Prop:new(x, y, attributes)
  Prop.super.new(self, x, y)
  
  table.insert(drawTable, self)
  self.type = "prop"
  self.isProp = true --Identifier to recognise children

  self.attributes = attributes or {}
  
  createAttribute(self.attributes, "name", "")
  createAttribute(self.attributes, "textureName", "placeholder")
  createAttribute(self.attributes, "draw", true)
  createAttribute(self.attributes, "rotation", 0)
  createAttribute(self.attributes, "protected", false)
  createAttribute(self.attributes, "scaleX", 1)
  createAttribute(self.attributes, "scaleY", 1)
  createAttribute(self.attributes, "animation", "")
  createAttribute(self.attributes, "painted", false)
  createAttribute(self.attributes, "priorityDraw", false)

  self.texture = texture[self.attributes.textureName]
  
  self:setDimentions()
  
  self.interactPromptOffset = 30
  
  --Decides if the texture or animation should be used, or both
  --0 - both
  --1 - texture only
  --2 - animation only
  
  self.drawMode = 0
  
  if self.attributes.name ~= "" then
    nameIndex:addEntity(self)
  end

end


function Prop:updateAnimation(dt)


  if self.attributes.animation == "" then
    self.animPlayer = nil
  else
    if not self.animPlayer then
      self.animPlayer = AnimationPlayer(animation[self.attributes.animation])
    end
    
    self.animPlayer:update(dt)
  end
end


function Prop:setPosition(x, y)
  self.x = x
  self.y = y
end


function Prop:delete()
  self.remove = true
  if self.body then
    self.body:destroy()
  end
  
end


function Prop:update(dt)
  self:updateAnimation(dt)
end



function Prop:checkSelect()
  
  self:setDimentions()
  
  return isPointInRectangle(
    mouse.worldX, 
    mouse.worldY, 
    self.x, 
    self.y, 
    self.width, 
    self.height)
end


function Prop:setDrawMode(mode)
  if not (self.drawMode == mode) then
    if mode == "both" or mode == nil then
      self.drawMode = 0
    elseif mode == "texture" then
      self.drawMode = 1
    elseif mode == "animation" then
      self.drawMode = 2
    else
      error("Unknown draw mode: ".. tostring(mode))
    
    end
  end
end


function Prop:updateBuilder()
  if builder:isModeProp() then
    if love.keyboard.isScancodeDown("delete") then
      if isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width, self.height) then
        if not self.attributes.protected then
          self:delete()
        end
      end
    end
  end
  if builder.selectedObject == self then
    --Selected but not finalised
    self.selected = true

    if love.mouse.isDown(1) and not self.attributes.protected then
      if isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width, self.height) and not self.grabbed then
        self.grabbed = true
        builder.grabOffsetX = mouse.worldX - self.x
        builder.grabOffsetY = mouse.worldY - self.y
      end
    else
      self.grabbed = false
    end

  end
  if self.grabbed then
    
    local offsetX = self.width%2 == 1
    local offsetY = self.height%2 == 1

    self:setPosition(lockToPixels(mouse.worldX - builder.grabOffsetX, mouse.worldY - builder.grabOffsetY, "small", offsetX, offsetY))

  end
end



function Prop:draw()
  self:standardPropDraw()
end


function Prop:drawOverlay()
  self:standardPropDrawOverlay()
end


function Prop:standardPropDraw()
  
  
  if self.attributes.draw then
    if self.attributes.textureName then
      self.texture = texture[self.attributes.textureName]
    else
      self.texture = texture[self.internalTextureName]
    end
  
    blue.graphics.setZ(-10)
  
    local rotation = math.rad(tonumber(self.attributes.rotation))
    if self.drawMode == 1 or self.drawMode == 0 then
      
      if self.texture.main then
        
        if self.attributes.painted then
          love.graphics.setColor(1,1,1,0.7)
          blue.graphics.drawCentred(self.texture, self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
        else
          love.graphics.setColor(1,1,1)
          blue.graphics.drawCentredShaded(self.texture, self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
        end
      end
    end
    
    
    if self.drawMode == 2 or self.drawMode == 0 then
      if self.attributes.animation and self.attributes.animation ~= "" then
        if not self.animPlayer then
          
          self.animPlayer = AnimationPlayer(animation[self.attributes.animation])
        else
          self.animPlayer.animation = animation[self.attributes.animation]
        end 
          
        love.graphics.setColor(1,1,1)
        self.animPlayer:play(self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
      end
    end

  end

  
  
end


function Prop:standardPropDrawOverlay()
  
  if (builderMode or debugMode) and ((not self.attributes.draw) or (self.attributes.textureName == "" and self.attributes.animation == "")) then
    love.graphics.setColor(1,0.5,0.5)
    blue.graphics.circle("fill", self.x, self.y, 10)
    blue.graphics.circle("line", self.x, self.y, 15)
  end
  
  
  
  if builderMode then
    if self.selected and builderMode then
      
      local width  = self.width
      local height = self.height
      
      love.graphics.setColor(1, 1, 1)
      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - width/2, self.y - height/2, width, height)
      love.graphics.setLineWidth("1")
    end
  end

  if debugMode or (self.selected and builderMode) then
    if self.attributes.interactRadius and self:isInteractable() then
      love.graphics.setColor(1,0,0)
      blue.graphics.circle("line", self.x, self.y, self.attributes.interactRadius)
    end
  end
  
  if self.active ~= false then
    self:drawInteractPrompt()
  end
end


function Prop:setDimentions()
  
  if self.texture.main then
    self.width = self.texture.main:getWidth()*smallTextureScale*math.abs(self.attributes.scaleX or 1)
    self.height = self.texture.main:getHeight()*smallTextureScale*math.abs(self.attributes.scaleY or 1)
  else
    self.width = self.texture:getWidth()*smallTextureScale*math.abs(self.attributes.scaleX or 1)
    self.height = self.texture:getHeight()*smallTextureScale*math.abs(self.attributes.scaleY or 1)
  end
  
end



function Prop:drawInteractPrompt()
  if self.nearest then
    
    love.graphics.setColor(1,1,1)
    
    local x = self.x
    local y = self.y - (self.interactPromptOffset or 30)
    
    
    if gamepadMode then
      local scale = smallTextureScale*0.75
      local width = font:getWidth(controls.interact.name)
      local height = font:getHeight()

      blue.graphics.print(controls.interact.name, x, y, 0, scale, scale, width/2, height/2)
      
    else
      local offset = 8
      blue.graphics.displayKey(x - offset, y - offset, controls.interact.name, 0.5)

      
    end
    self.nearest = nil
  end
end
