raindrops = {}

local textureWidth
local textureHeight

function createRaindropGroup()
  raindrops.members = {}
  raindrops.type = "raindrops"
  raindrops.spriteBatch = love.graphics.newSpriteBatch(texture.raindrop, 1000)
  textureWidth = texture.raindrop:getWidth()
  textureHeight = texture.raindrop:getHeight()
  table.insert(drawTable, raindrops)
end


function raindrops:draw()
  
  self.spriteBatch:clear()
  
  for i, raindrop in ipairs(self.members) do
    
    local velocity = vector.getMagnitude(Vec2(raindrop.body:getLinearVelocity()))
    
    local stretch = math.max(velocity*0.005, 0.5)
    
    local r
    if velocity > 2 then
      r = vector.getOrientation(Vec2(raindrop.body:getLinearVelocity()))
    else
      r = math.rad(90)
    end
    
    self.spriteBatch:add(raindrop.body:getX(), raindrop.body:getY(), r, smallTextureScale, smallTextureScale*stretch, textureWidth/2, textureHeight/2)
  end
  
  love.graphics.setColor(1,1,1)
  blue.graphics.draw(raindrops.spriteBatch, 0, 0)
end


function raindrops:update(dt)
  for i, raindrop in ipairs(self.members) do
    
    if raindrop.life <= 0 then
      table.remove(raindrops.members, i)
      raindrop.body:destroy()
    else
      
      if not effectedByTimeBubble(raindrop) then
        raindrop.life = raindrop.life - dt
      end
    end
  end
end



Raindrop = Object:extend()

function Raindrop:new(x, y, life, vx, vy, damping)
  
  
  self.body = love.physics.newBody(world, x, y, "dynamic")
  self.body:setLinearVelocity(vx or 0, vy or 0)
  self.body:setFixedRotation(true)
  self.shape = love.physics.newCircleShape(0.1*smallTextureScale)
  self.fixture = love.physics.newFixture(self.body, self.shape, 0.01)
  self.fixture:setFriction(1)
  self.fixture:setRestitution(0.13)
  self.fixture:setCategory(2)
  self.fixture:setMask(2)
  self.body:setLinearDamping(damping or 0)
  
  self.life = life or 3
  Raindrop.super.new(self)

  
  
  table.insert(raindrops.members, self)
end





