sparks = {}

local textureWidth
local textureHeight

function createSparkGroup()
  sparks.members = {}
  sparks.type = "sparks"
  sparks.spriteBatch = love.graphics.newSpriteBatch(texture.spark, 1000)
  textureWidth = texture.spark:getWidth()
  textureHeight = texture.spark:getHeight()
  table.insert(drawTable, sparks)
end


function sparks:draw()
  
  self.spriteBatch:clear()
  
  for i, spark in ipairs(self.members) do
    self.spriteBatch:add(spark.body:getX() - textureWidth/2, spark.body:getY() - textureHeight/2)
  end
  
  love.graphics.setColor(1,1,1)
  blue.graphics.drawShaded(sparks.spriteBatch, 0, 0)
end


function sparks:update(dt)
  for i, spark in ipairs(self.members) do
    
    if spark.life <= 0 then
      table.remove(sparks.members, i)
      spark.body:destroy()
    else
      
      if not effectedByTimeBubble(spark) then
        spark.life = spark.life - dt
      end
    end
  end
end





Spark = Object:extend()

function Spark:new(x, y, life, vx, vy, damping)
  
  
  self.body = love.physics.newBody(world, x, y, "dynamic")
  self.body:setLinearVelocity(vx, vy)
  self.body:setFixedRotation(true)
  self.shape = love.physics.newCircleShape(1)
  self.fixture = love.physics.newFixture(self.body, self.shape, 0.01)
  self.fixture:setRestitution(0)
  self.fixture:setCategory(2)
  self.fixture:setMask(2)
  self.body:setLinearDamping(damping or 4)
  
  self.life = life or 1
  Spark.super.new(self)

  
  
  table.insert(sparks.members, self)
end





