LogicProp = Prop:extend()

local logic = {}


function LogicProp:new(x, y, attributes)
  LogicProp.super.new(self, x, y, attributes)
  self.type = "logicProp"

  self.attributes.textureName = nil
  self.attributes.animation = nil
  self.attributes.scaleX = nil
  self.attributes.scaleY = nil
  self.attributes.draw = nil
  self.attributes.priorityDraw = nil
  self.attributes.painted = nil
  self.attributes.rotation = nil
  
  createAttribute(self.attributes, "logicType", "")
  createAttribute(self.attributes, "triggerTarget", "")
  createAttribute(self.attributes, "colour", "")
  createAttribute(self.attributes, "maxTriggers", 0)
  createAttribute(self.attributes, "arg1", "")
  createAttribute(self.attributes, "arg2", "")
  createAttribute(self.attributes, "arg3", "")
  createAttribute(self.attributes, "arg4", "")
  
  self.numberOfTriggers = 0
  
end



function LogicProp:activate(inputVector, ...)
  
  local args = {...}
  
  if self.attributes.maxTriggers <= 0 or self.numberOfTriggers < self.attributes.maxTriggers then
    if self.attributes.logicType == "" or (not logic[self.attributes.logicType]) then
      self:trigger(inputVector)
    else
      logic[self.attributes.logicType](self, inputVector, ...)
    end
  end
end



function LogicProp:trigger(inputVector, ...)
  
  local targets = nameIndex.entities[self.attributes.triggerTarget]
  
  self.numberOfTriggers = self.numberOfTriggers + 1
  
  if targets then
    for i, target in ipairs(targets) do
      if target.activate then
        target:activate(inputVector, ...)
      end
    end
  end
end


function LogicProp:draw()
  if builderMode then

    
    if #self.attributes.colour == 6 then
      local hexCode = self.attributes.colour
      
      shader.setColour:send("setColour", {
          tonumber(hexCode:sub(1, 2),16)/255,
          tonumber(hexCode:sub(3, 4),16)/255, 
          tonumber(hexCode:sub(5, 6),16)/255})
      
      love.graphics.setShader(shader.setColour)
      blue.graphics.drawCentred(texture.logic_prop, self.x, self.y, 0, smallTextureScale, smallTextureScale)
      love.graphics.setShader()
    else
      blue.graphics.drawCentred(texture.logic_prop, self.x, self.y, 0, smallTextureScale, smallTextureScale)
    end
    
    
    if self.selected then
      love.graphics.setColor(1, 1, 1)
      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - self.width/2, self.y - self.height/2, self.width, self.height)
      love.graphics.setLineWidth("1")
    end
  end
end




function logic.addToTwo(self, inputVector, value)
  
  if not self.counter then
    self.counter = 0
  end
  
  self.counter = self.counter + value
  if self.counter == 2 then
    self:trigger(inputVector)
  end
  
end