--My modifications to the standard graphics library
--Intended purpose is to facilitate custom transformations of the whole scene
blue = {}
blue.graphics = {}
local pixelWarning = false

local lightHandler = require "lighting/lightHandler"

local passes = require "rendering"


-- Transformation to apply to most items in scene

function blue.standardXTransform(x)
  x = x*camera.zoom + (camera.x*camera.zoom + window.width/2)
  return x
end

function blue.standardYTransform(y)
  y = y*camera.zoom + (camera.y*camera.zoom + window.height/2)
  return y
end


function blue.standardTransform(x, y)
  x = blue.standardXTransform(x)
  y = blue.standardYTransform(y)
  return x, y
end

function blue.graphics.rectangle(mode, x, y, width, height)
  x, y = blue.standardTransform(x, y)
  
  -- Note - most of these function do not take line width into account
  
  if blue.graphics.testIfOnscreen(x, y, width*2*camera.zoom, height*2*camera.zoom) then
    love.graphics.rectangle(mode, x, y, width*camera.zoom, height*camera.zoom)
  end
end


function blue.graphics.circle(mode, x, y, radius)

  x, y = blue.standardTransform(x, y)
  if blue.graphics.testIfOnscreen(x, y, radius*2*camera.zoom, radius*2*camera.zoom) then
    love.graphics.circle(mode, x, y, radius*camera.zoom)
  end
end

function blue.graphics.line(x1, y1, x2, y2)
  x1, y1 = blue.standardTransform(x1, y1)
  x2, y2 = blue.standardTransform(x2, y2)
  
  love.graphics.line(x1, y1, x2, y2)
end

function blue.graphics.parallaxCircle(mode, x, y, z, radius)
  x = ((x + camera.x)*camera.zoom)/z + window.width/2
  y = ((y + camera.y)*camera.zoom)/z + window.height/2
  if blue.graphics.testIfOnscreen(x, y, radius*2*camera.zoom, radius*2*camera.zoom) then
    love.graphics.circle(mode, x, y, radius*camera.zoom)
  end
end

function blue.graphics.parallaxEllipse(mode, x, y, z, radiusX, radiusY)
  x = (x + camera.x)/z + window.width/2
  y = (y + camera.y)/z + window.height/2
  if blue.graphics.testIfOnscreen(x, y, radiusX*2*camera.zoom, radiusY*2*camera.zoom) then
    love.graphics.ellipse(mode, x, y, radiusX*camera.zoom, radiusY*camera.zoom)
  end
end


function blue.graphics.parallaxDraw(drawable, x, y, z, r, sx, sy, ox, oy, kx, ky)
  
  if not drawable then
    error("Expected drawable, got nil!", 2)
  end

  if z == 0 then
    --print("WARNING - Z WAS 0. SETTING TO 1")
    z = 1
  end
  
  x = ((x + camera.x)*camera.zoom)/z + window.width/2
  y = ((y + camera.y)*camera.zoom)/z + window.height/2
  
  local textureWidth = drawable:getWidth()*sx/z
  local textureHeight = drawable:getHeight()*sy/z
  

  if kx or ky then
    print("WARNING! parallaxDraw has not been modified to take sheer into account!")
  end
  ox = ox or 0
  oy = oy or 0
  sx = sx or 1
  sy = sy or sx
  
  
  if blue.graphics.testIfOnscreen(((x + ox) + textureWidth/2*camera.zoom), (y + oy) + textureHeight/2*camera.zoom, textureWidth*camera.zoom, textureHeight*camera.zoom) then
    love.graphics.draw(drawable, x, y, r, sx/z*camera.zoom, sy/z*camera.zoom, 0, 0, kx, ky)
  end
end


function blue.graphics.draw(drawable, x, y, r, sx, sy, ox, oy, kx, ky) --TODO - fix onscreen test for images that rotare
  
  if not drawable then
    error("Expected drawable, got nil!", 2)
  end
  
  x, y = blue.standardTransform(x, y)
  ox = ox or 0
  oy = oy or 0
  sx = sx or 1
  sy = sy or sx
  
  local onScreen
  
  
  if drawable:type() == "Mesh" then
    onScreen = true
  elseif drawable:type() ~= "SpriteBatch" then
    local textureWidth = drawable:getWidth()*sx
    local textureHeight = drawable:getHeight()*sy


    onScreen = (not(r == 0 or not r)) or blue.graphics.testIfOnscreen(((x + ox) + textureWidth/2*camera.zoom), (y + oy) + textureHeight/2*camera.zoom, textureWidth*camera.zoom, textureHeight*camera.zoom) 
  else
    onScreen = true
  end

  
  
  if onScreen then
    love.graphics.draw(drawable, x, y, r, sx*camera.zoom, sy*camera.zoom, ox, oy, kx, ky)
  end

end



function blue.graphics.drawCentred(drawable, x, y, r, sx, sy, ox, oy, kx, ky)
  
  x, y = blue.standardTransform(x, y)

  ox = ox or 0
  oy = oy or 0
  sx = sx or 1
  sy = sy or sx

  if drawable.main then
    drawable = drawable.main
  end
  
  local textureWidth = math.abs(drawable:getWidth()*sx)
  local textureHeight = math.abs(drawable:getHeight()*sy)
  
  local testX = (x + ox - textureWidth/2*camera.zoom) + textureWidth/2*camera.zoom
  local testY = (y + oy - textureHeight/2*camera.zoom) + textureHeight/2*camera.zoom
  local testWidth = textureWidth*camera.zoom+ox*2
  local testHeight = textureHeight*camera.zoom+oy*2
  
  if blue.graphics.testIfOnscreen(testX, testY, testWidth, testHeight) then
    love.graphics.draw(drawable, x, y, r, sx*camera.zoom, sy*camera.zoom, ox + drawable:getWidth()/2, oy + drawable:getHeight()/2, kx, ky)
  end
  
end



function blue.graphics.polygon(mode, pointTable)
  
  if pointTable[1] == pointTable[#pointTable-1] and pointTable[2] == pointTable[#pointTable] then
    table.remove(pointTable, #pointTable)
    table.remove(pointTable, #pointTable)
  end
  
  local newPos = {}
  for i, v in ipairs(pointTable) do
  
    if i%2 == 1 then
      table.insert(newPos, blue.standardXTransform(v))
    else
      table.insert(newPos, blue.standardYTransform(v))
    end
  end
  --Convex correction
  
  if not love.math.isConvex(newPos) and mode == "fill" then
    local triangles = love.math.triangulate(newPos)
    for i, triangle in ipairs(triangles) do
      love.graphics.polygon(mode, triangle)

    end
  else
    love.graphics.polygon(mode, newPos)
  end
end

function blue.graphics.testIfOnscreen(x, y, width, height)
  
  --Important note - tests based on the centre of the rectangle, thus requires adjustment to work with functions that start in the corner
  
  --width = width --* camera.zoom
  --height = height-- *camera.zoom
  
  r, g, b, a = love.graphics.getColor()
  
  if debugMode and false then
    love.graphics.setColor(0,0,0,1)
    love.graphics.rectangle("line", x - width/2, y - height/2, width, height)
    love.graphics.setColor(r, g, b, a)
  end
  
  if window.width < x - width/2 or
  0 > x + width/2 or
  window.height < y - height/2 or
  0 > y + height/2
  then
    return false
  else
    return true
  end
  
end

function blue.graphics.vectorLine(x, y, v, length)
  length = length or 1
  blue.graphics.line(x, y, x + v.x*length, y + v.y*length)
end



function blue.graphics.print(text, x, y, rot, sx, sy, ox, oy, kx, ky)
  
  x, y = blue.standardTransform(x, y)
  sy = sy or sx

  love.graphics.print(text, x+1, y+1, rot, sx*camera.zoom, sy*camera.zoom, ox, oy, kx, ky)

end


function blue.graphics.printShadow(text, x, y, rot, sx, sy, ox, oy, kx, ky)

  r, g, b, a = love.graphics.getColor()
  a = 1
  if r == 0 and g == 0 and b == 0 then
    love.graphics.setColor(1, 1, 1, a*0.5) 
  else
    love.graphics.setColor(0, 0, 0) 
  end
  love.graphics.print(text, x+1, y+1, rot, sx, sy, ox, oy, kx, ky)
  love.graphics.setColor(r, g, b, a) 
  love.graphics.print(text, x, y, rot, sx, sy, ox, oy, kx, ky)
  
end




function blue.graphics:wrappedDraw(drawable, x, y, r, scale)
  
  if not x then
    error("x was not given!", 2)
  elseif not y then
    error("y was not given!", 2)
  end
  
  r = r or 0
  scale =  scale or 1
  x, y = blue.standardTransform(x, y)
  
  drawable:setWrap( "repeat", "repeat" )
  
  local quad = love.graphics.newQuad( -x, -y, window.width, window.height, drawable:getWidth()*camera.zoom*scale, drawable:getHeight()*camera.zoom*scale )
  love.graphics.draw( drawable, quad, 0, 0, r, 1, 1, 0, 0)
end



function blue.graphics.drawCentredSet(set, x, y, r, sx, sy, ox, oy, kx, ky)

  blue.graphics.drawCentred(set.main, x, y, r, sx, sy, ox, oy, kx, ky)
  

  local r, g, b, a = love.graphics.getColor()
  
  --Disables the energy shader
  love.graphics.setShader()
  
  if set.shadow then
    love.graphics.setColor(0, 0, 0, scene.lightHarshness)
    blue.graphics.drawCentred(set.shadow, x, y, r, sx, sy, ox, oy, kx, ky)
  end

  love.graphics.setColor(r, g, b, a)
  
end

local z = 0

function blue.graphics.setZ(inZ)
  z = inZ
end

function blue.graphics.getZ()
  return z
end





function blue.graphics.drawCentredShaded(set, x, y, r, sx, sy, ox, oy, kx, ky)
  
  
  shader.renderPasses:send("depthValue", blue.graphics.getZ())
  if set.normalMap then
    shader.renderPasses:send("rotation", r)
    shader.renderPasses:send("flipX", sx < 0)
    shader.renderPasses:send("flipY", sy < 0)
    shader.renderPasses:send("hasNormalMap", true)
    shader.renderPasses:send("normalMap", set.normalMap)
  else
    shader.renderPasses:send("hasNormalMap", false)
  end
  
  love.graphics.setShader(shader.renderPasses)
  blue.graphics.drawCentred(set, x, y, r, sx, sy, ox, oy, kx, ky)
  love.graphics.setShader()
end


function blue.graphics.drawShaded(set, x, y, r, sx, sy, ox, oy, kx, ky)
  
  local drawable = set.main or set

  shader.renderPasses:send("depthValue", blue.graphics.getZ())
  if set.normalMap then
    shader.renderPasses:send("rotation", r)
    shader.renderPasses:send("flipX", sx < 0)
    shader.renderPasses:send("flipY", sy < 0)
    shader.renderPasses:send("hasNormalMap", true)
    shader.renderPasses:send("normalMap", set.normalMap)
  else
    shader.renderPasses:send("hasNormalMap", false)
  end
  love.graphics.setShader(shader.renderPasses)
  blue.graphics.draw(drawable, x, y, r, sx, sy, ox, oy, kx, ky)
  love.graphics.setShader()

  
end




function blue.graphics.triangle(mode, x, y, r, radius, thickness)
  
  x, y = blue.standardTransform(x, y)
  
  if blue.graphics.testIfOnscreen(x, y, radius*2*camera.zoom, radius*2*camera.zoom) then
    
    thickness = thickness or 1
    local originalThickness = love.graphics.getLineWidth()
    
    radius = radius * camera.zoom
    local cosR = math.cos(r)
    local sinR = math.sin(r)
    
    local p1 = Vec2(-1  * radius,  0)
    local p2 = Vec2(0.5 * radius,  0.8660254038 * radius)
    local p3 = Vec2(0.5 * radius, -0.8660254038 * radius)
    
    local function rotate(v, cosR, sinR)
      local returnV = {}
      
      returnV.x = v.x * cosR  -  v.y * sinR
      returnV.y = v.y * cosR  +  v.x * sinR
      
      return returnV
    end
    
    p1 = rotate(p1, cosR, sinR)
    p2 = rotate(p2, cosR, sinR)
    p3 = rotate(p3, cosR, sinR)
    
    local points = {
      p1.x+x, p1.y+y,
      p2.x+x, p2.y+y,
      p3.x+x, p3.y+y}
    
    
    love.graphics.setLineWidth(thickness*camera.zoom)
    
    love.graphics.polygon(mode, points)
    
    love.graphics.setLineWidth(originalThickness)

  end
  
end


function love.graphics.triangle(mode, x, y, r, radius, thickness)
  
  thickness = thickness or 1
  local originalThickness = love.graphics.getLineWidth()
  

  local cosR = math.cos(r)
  local sinR = math.sin(r)
  
  local p1 = Vec2(-1  * radius,  0)
  local p2 = Vec2(0.5 * radius,  0.8660254038 * radius)
  local p3 = Vec2(0.5 * radius, -0.8660254038 * radius)
  
  local function rotate(v, cosR, sinR)
    local returnV = {}
    
    returnV.x = v.x * cosR  -  v.y * sinR
    returnV.y = v.y * cosR  +  v.x * sinR
    
    return returnV
  end
  
  p1 = rotate(p1, cosR, sinR)
  p2 = rotate(p2, cosR, sinR)
  p3 = rotate(p3, cosR, sinR)
  
  local points = {
    p1.x+x, p1.y+y,
    p2.x+x, p2.y+y,
    p3.x+x, p3.y+y}
  
  
  love.graphics.setLineWidth(thickness*camera.zoom)
  
  love.graphics.polygon(mode, points)
  
  love.graphics.setLineWidth(originalThickness)

  
end



function love.graphics.displayKey(x, y, letter, scale)
  
  love.graphics.setColor(1,1,1)
  scale = scale or 1

  
  love.graphics.draw(texture.keypress, x, y, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  love.graphics.setColor(0.8,0.8,0.8)
  love.graphics.print(letter, x + 9*smallTextureScale*scale, y + 3.5*smallTextureScale*scale, 0, smallTextureScale*scale, smallTextureScale*scale)

end

function love.graphics.displayCentredKey(x, y, letter, scale)
  
  love.graphics.setColor(1,1,1)
  scale = scale or 1
  
  local offsetX = - texture.keypress:getWidth()/2 * scale * smallTextureScale
  local offsetY = - texture.keypress:getHeight()/2 * scale * smallTextureScale
  
  love.graphics.draw(texture.keypress, x + offsetX, y + offsetY, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  love.graphics.setColor(0.8,0.8,0.8)
  love.graphics.print(letter, x + 9*smallTextureScale*scale + offsetX, y + 3.5*smallTextureScale*scale + offsetY, 0, smallTextureScale*scale, smallTextureScale*scale)

end


function blue.graphics.displayKey(x, y, letter, scale)
  
  scale = scale or 1
  blue.graphics.draw(texture.keypress, x, y, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  love.graphics.setColor(0.8,0.8,0.8)
  blue.graphics.print(letter, x + 9*smallTextureScale*scale, y + 3.5*smallTextureScale*scale, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  
end



function blue.getPolygonDimentions(pointTable)
  --Set lowest x and y values to 0, and find the offset
  local newPoints = {}
  local lowestX = pointTable[1]
  local lowestY = pointTable[2]
  local highestX = pointTable[1]
  local highestY = pointTable[2]
  
  for i, coord in ipairs(pointTable) do
    if i > 2 then
      if i%2 == 1 then
        --X
        if coord < lowestX then
          lowestX = coord
        elseif coord > highestX then
          highestX = coord
        end
      else
        --Y
        if coord < lowestY then
          lowestY = coord
        elseif coord > highestY then
          highestY = coord
        end
      end
    end
  end
  
  local width = highestX - lowestX
  local height = highestY - lowestY
  return lowestX, lowestY, width, height
end



local quad = love.graphics.newQuad( 0, 0, 0, 0, 1, 1)

function blue.graphics.texturedPolygon(drawable, points, scale, extraX, extraY)
  
  local lowestX, lowestY, width, height = blue.getPolygonDimentions(points)
  local testX, testY = blue.standardTransform(lowestX + extraX, lowestY + extraY)
  
  if blue.graphics.testIfOnscreen((testX + width/2*camera.zoom), testY + height/2*camera.zoom, width*camera.zoom, height*camera.zoom) then
  
    love.graphics.push()
    
    x, y = blue.standardTransform(0, 0)
    
    love.graphics.translate(x, y)
    love.graphics.scale(camera.zoom, camera.zoom)

    love.graphics.stencil(
      function()
        if not love.math.isConvex(points) then
          local triangles = love.math.triangulate(points)
          for i, triangle in ipairs(triangles) do
            love.graphics.polygon("fill", triangle)

          end
        else
          love.graphics.polygon("fill", points)
        end
      end
    , "replace", 1)
    
    love.graphics.setStencilTest("greater", 0)

    quad:setViewport(0, 0, width, height, scale*drawable:getWidth(), scale*drawable:getHeight())
    drawable:setWrap("repeat", "repeat")
    
    love.graphics.draw(drawable, quad, lowestX, lowestY, 0, 1, 1)
    love.graphics.setStencilTest()
    love.graphics.pop()
  end
end

