
--vector maths 3.0
vector = {}

function vector.create(x, y)
  local v = {}
  v.x = x
  v.y = y or x
  setmetatable(v, vector.mt)
  return v
end

Vec2 = vector.create

function vector.turn90Degrees(v)
  return Vec2(v.y, v.x*-1)
end


function vector.simplify(v)
  --Turns a vector into a table with two variables, but no keys
  local nv = {}
  nv[1] = v.x
  nv[2] = v.y
  return nv
end

function vector.project(a, b)
  --Project a onto b
  return vector.scalarMultiplication(b, vector.dotProduct(a, b))
end


function vector.dotProduct(a, b)
  return a.x * b.x + a.y * b.y
end

vector.dot = vector.dotProduct


function vector.addition(a, b)
  
  if type(a) == "table" and type(b) == "table" then
    return Vec2(a.x + b.x, a.y + b.y)
  else
    local vec
    local scalar
    
    if type(a) == "number" then
      scalar = a
      vec = b
    else
      scalar = b
      vec = a
    end
    
    return Vec2(vec.x + scalar, vec.y + scalar)
  end
end

function vector.subtraction(a, b)
  if type(a) == "table" and type(b) == "table" then
    return Vec2(a.x - b.x, a.y - b.y)
  else

    if type(a) == "number" then
      return Vec2(b.x - a, b.y - a)
    else
      return Vec2(a.x - b, a.y - b)
    end
  end
end

function vector.negation(v)
  return Vec2(-v.x, -v.y)
end

function vector.isParallel(a, b)
  
  return (a.x == 0 and b.x == 0) or (a.y == 0 and b.y == 0) or (a.x/b.x == a.y/b.y)
  
end


function vector.isEqual(a, b)
  return a.x == b.x and a.y == b.y
end


function vector.mult(a, b) 
  if type(a) == "number" then
    return vector.scalarMultiplication(b, a)
  else
    return vector.scalarMultiplication(a, b)
  end
end

function vector.div(a, b) 
  return vector.scalarDivision(a, b)
end

function vector.scalarMultiplication(v, s)
  return Vec2(v.x * s, v.y * s)
end

function vector.scalarDivision(v, s)
  return Vec2(v.x / s, v.y / s)
end


function vector.getMagnitude(v)
  return math.sqrt(v.x^2 + v.y^2)
end


function vector.getMagnitudeSqrd(v)
  return v.x^2 + v.y^2
end


function vector.normalize(v)
  local vectorMagnitune = vector.getMagnitude(v)
  local x
  local y
  if vectorMagnitune ~= 0 then
    x = v.x / vectorMagnitune
    y = v.y / vectorMagnitune
  else
    x = 0
    y = 0
  end
  return Vec2(x, y)
end

function vector.getOrientation(v)
  local verticalVector = vector.create(0, 1)
  local tempV = vector.create(v.x, v.y)
  
  if v.x > 0 then
    return math.rad(360) - math.acos(vector.dotProduct(tempV, verticalVector)/vector.getMagnitude(tempV))
  else
    return math.acos(vector.dotProduct(tempV, verticalVector)/vector.getMagnitude(tempV))
  end
end


function vector.angleBetween(v1, v2)
  local denominator = (vector.getMagnitude(v1)*vector.getMagnitude(v2))

  return math.acos(math.min(vector.dotProduct(v1, v2)/denominator, 1))
end


function vector.interpolate(v1, v2, t)
  
  local difference = v2 - v1
  return v1 + difference*t
  
end



function vector.angleToVector(angle)
  return Vec2(math.sin(angle) * -1, math.cos(angle))
end


function vector.angleIsClockwise(a, b)
  
  --[[-- Net normals to v1 on both sides
  local n1 = vector.normalize(vector.create(v1.y, v1.x*-1))
  local n2 = vector.scalarMultiplication(n1, -1)
  
  -- Get the displacement between the mouse and points on the normals
  local d1 = v2 - n1
  local d2 = v2 - n2
  
  -- Whichever normal is closer
  return d1 > d2 ]]
  
  --Wow this one is a lot simpler
  
  return a.x * b.y - a.y * b.x > 0
  
  
  
end

vector.mt = {}
vector.mt.__add = vector.addition
vector.mt.__sub = vector.subtraction
vector.mt.__unm = vector.negation
vector.mt.__eq  = vector.isEqual
vector.mt.__le = 
function(a, b) 
  return vector.getMagnitudeSqrd(a) <= vector.getMagnitudeSqrd(b) 
end

vector.mt.__lt = 
function(a, b) 
  return vector.getMagnitudeSqrd(a) < vector.getMagnitudeSqrd(b) 
end

vector.mt.__tostring = function(v) return "("..v.x..", "..v.y..")" end
vector.mt.__mul = vector.mult
vector.mt.__div = vector.div
