local handler = {lights = {}}

--This probably isn't the most efficient method as lights are constantly being added and deleted, but it's the best I could come up with


function addLight(x, y, z, intensity, colour)
  
  
  local newLight = {}
  
  newLight.x = x
  newLight.y = y
  newLight.z = z
  
  
  if intensity == 0 then
    error("Cannot set light to an intensity of 0!", 2)
  end
  
  newLight.intensity = intensity
  newLight.colour = colour or {1,1,1}
  
  table.insert(handler.lights, newLight)
  
end


local aColour
function handler:setAmbient(intensity, input_colour)
  
  input_colour = input_colour or {1,1,1}
  
  local colour = {}
  colour[1] = input_colour[1] * intensity
  colour[2] = input_colour[2] * intensity
  colour[3] = input_colour[3] * intensity

  
  aColour = colour
  
  shader.lambertLighting:send("ambientLight", colour)
end


function handler:getAmbient()
  return aColour
end


function handler:clearLights()
  
  handler.lights = {}
  
end


function handler:update()
  self:sendLights()
  self:clearLights()
end


local function sendToShader(shader, lightPositions, lightColours, lightIntensities)
  
  
  shader:send("lightIntensities", unpack(lightIntensities))
  
  
  if #lightPositions > 0 then
    shader:send("cameraZoom", camera.zoom)
    shader:send("lights", unpack(lightPositions))
    shader:send("lightColours", unpack(lightColours))
  end

end


function handler:sendLights()
  
  local maxLights = 64
  
  local lightPositions = {}
  local lightColours = {}
  local lightIntensities = {}
  
  for i = 1, maxLights do
    table.insert(lightIntensities, 0)
  end
  
  for i, light in ipairs(self.lights) do
    
    --Positions need to be traslated
    
    local x, y = blue.standardTransform(light.x, light.y)
    
    
    table.insert(lightPositions, {x, y, light.z})
    
    lightIntensities[i] = light.intensity
    table.insert(lightColours, light.colour)
    
  end
  
  --WARNING - a light of 0 intensity will disable all following lights 
  
  sendToShader(shader.lambertLighting, lightPositions, lightColours, lightIntensities)
end





return handler