local function convertToImage(vectorTable, imageData)
  
  local width = imageData:getWidth()
  local height = imageData:getHeight()
  
  local newNormalMap = love.graphics.newCanvas(width, height)
  
  
  love.graphics.setCanvas(newNormalMap)
  for x, row in ipairs(vectorTable) do
    x = x - 1
    for y, value in ipairs(row) do
      y = y - 1
      
      if select(4, imageData:getPixel(x, y)) > 0 then
      
        local r = value.x
        local g = value.y
        local b = value.z

        
        love.graphics.setColor(r, g, b)
        
        love.graphics.points(x+0.5, y+0.5)
        
      end
    end
  end
  love.graphics.setCanvas()
  
  
  
  return newNormalMap
end


local function getNormal(imageData, x, y, normal, distance, weight)
  
  local width = imageData:getWidth()
  local height = imageData:getHeight()
  
  --Left
  if x + distance >= width or select(4, imageData:getPixel(x + distance, y)) == 0 then
    normal.x = normal.x + weight
  end
  --right
  if x - distance < 0 or select(4, imageData:getPixel(x - distance, y)) == 0 then
    normal.x = normal.x - weight
  end
  --up
  if y - distance < 0 or select(4, imageData:getPixel(x, y - distance)) == 0 then
    normal.y = normal.y - weight
  end
  --down
  if y + distance >= height or select(4, imageData:getPixel(x, y + distance)) == 0 then
    normal.y = normal.y + weight
  end
  
  --[[
  
  
  local diagonalWeight = math.sqrt(distance^2 + distance^2)
  
  --Left up
  if x + distance >= width or y + distance >= height or select(4, imageData:getPixel(x + distance, y + distance)) == 0 then
    normal.x = normal.x + weight * diagonalWeight
    normal.y = normal.y + weight * diagonalWeight
  end
  --right up
  if x - distance < 0 or y + distance >= height or select(4, imageData:getPixel(x - distance, y + distance)) == 0 then
    normal.x = normal.x - weight * diagonalWeight
    normal.y = normal.y + weight * diagonalWeight
  end
  --Left down
  if x + distance >= width or y - distance < 0 or select(4, imageData:getPixel(x + distance, y - distance)) == 0 then
    normal.x = normal.x + weight * diagonalWeight
    normal.y = normal.y - weight * diagonalWeight
  end
  --right down
  if x - distance < 0 or y - distance < 0 or select(4, imageData:getPixel(x - distance, y - distance)) == 0 then
    normal.x = normal.x - weight * diagonalWeight
    normal.y = normal.y - weight * diagonalWeight
  end
  
  ]]
  
  
  return normal
end


function newNormalMap(path)
  local imageData = love.image.newImageData(path)
  local vectorTable = {}
  
  for x = 0, imageData:getWidth()-1 do
    vectorTable[x+1] = {}
    
    for y = 0, imageData:getHeight()-1 do
      
      local normal = {}
      normal.x = 0
      normal.y = 0
      normal.z = 1
      
      
      local thisPixel = {imageData:getPixel(x, y)}
      
      if thisPixel[4] > 0 then
        normal = getNormal(imageData, x, y, normal, 1, 1)
        normal = getNormal(imageData, x, y, normal, 2, 0.5)
      end
      
      do
        local magnitude = math.sqrt(normal.x^2 + normal.y^2 + normal.z^2)
        normal.x = 0.5 + normal.x/magnitude/2
        normal.y = 0.5 + normal.y/magnitude/2
        normal.z = 0.5 + normal.z/magnitude/2
      end

      
      vectorTable[x+1][y+1] = normal
    end
  end
  
  return convertToImage(vectorTable, imageData)
end

