DialogueBox = Object:extend()

require "dialogue/characterInfo"


function DialogueBox:new(x, y, filename)
  self.type = "dialogueBox"
  self.x = x
  self.y = y

  self.fontScale = 2
  self.width = 1000
  self.height = font:getHeight()*self.fontScale*4 +20
  self.boarder = 10
  self.safeZoneX = 7
  
  self.pauseWorld = false
  
  self.page = {}
  self.character = {}
  if filename then
    self:loadBook(filename)
  end
  self.currentPage = 1
  
  
  self.timeDisplayed = false
  self.printSpeed = 35
  table.insert(drawTable, self)
  
  self.bookQueue = {}
end


function DialogueBox:updatePaused()
  if controls.interact.thisFrame and self.displayed then
    if #self.cutPage >= #self.page[self.currentPage] then
      self:nextPage()
    else
      self.timeDisplayed = -10000
    end
  end
end



function DialogueBox:drawOverlay()
  
  self.displayed = self.page[1] ~= nil
  if self.displayed then
  
    love.graphics.setColor(1,1,1)
    love.graphics.rectangle("fill", self.x - self.boarder - self.width/2 - self.safeZoneX, self.y - self.boarder - self.height/2, self.width + self.boarder*2 + self.safeZoneX*2, self.height + self.boarder*2)
    love.graphics.setColor(0,0,0)
    love.graphics.rectangle("fill", self.x - self.width/2 - self.safeZoneX, self.y - self.height/2, self.width + self.safeZoneX*2, self.height)
    love.graphics.setColor(1,1,1)
    
    local offsetX = 1 --(self.width - font:getWidth(self.page[self.currentPage])*self.fontScale)/2
    local _, lines = self.page[self.currentPage]:gsub('\n', '\n')
    local offsetY = (self.height - font:getHeight(self.page[self.currentPage])*self.fontScale*(lines ))/2
    
    local currentChar = (love.timer.getTime( ) - self.timeDisplayed)*self.printSpeed
    self.cutPage = self.page[self.currentPage]:specialSubstring(currentChar)
    
    if self.character[self.currentPage] then
      local safeZomeX = 7
      local safeZomeY = 5
      local text = self.character[self.currentPage]
      local width = font:getWidth(text)*self.fontScale + safeZomeX*2
      local height = font:getHeight()*self.fontScale + safeZomeY*2
      local boarderThickness = 10
      local x = self.x - self.width/2 - 50
      local y = self.y - self.height/2-height+1 -- boarderThickness
      
      local r, g, b = 1, 1, 1
      
      if characterInfo[self.character[self.currentPage]] then
        r, g, b = characterInfo[self.character[self.currentPage]]:getColour()
      end
      
      
      love.graphics.setColor(1,1,1)
      love.graphics.rectangle("fill", x-boarderThickness, y-boarderThickness, width + boarderThickness*2, height + boarderThickness*2)
      love.graphics.setColor(0,0,0)
      love.graphics.rectangle("fill", x, y, width, height)
      love.graphics.setColor(r, g, b)
      love.graphics.print(text, x + safeZomeX, y + safeZomeY, 0, self.fontScale, self.fontScale)
    end
    
    
    love.graphics.print(self.cutPage, self.x - self.width/2 + offsetX, self.y - self.height/2 + offsetY, 0, self.fontScale, self.fontScale)
    
    if currentChar >= #self.page[self.currentPage] then
      
      local x = self.x+self.width/2-20
      local y = self.y+70
      
      if gamepadMode then
        local scale = 4
        local width = font:getWidth(controls.interact.name)
        local height = font:getHeight()
        
        love.graphics.setColor(1,1,1)
        love.graphics.print(controls.interact.name, x + 2*smallTextureScale*texture.keypress:getWidth()/2, y + 2*smallTextureScale*texture.keypress:getHeight()/2, 0, scale, scale, width/2, height/2)
        

      else
        love.graphics.displayKey(x, y, controls.interact.name, 2)
      end

    end
  end
end


function DialogueBox:queueBook(filename)
  table.insert(self.bookQueue, filename)
  if not self.timeDisplayed then
    self:loadBook(filename)
  end
end


function DialogueBox:loadBook(filename, pauseWorld)
  
  
  if pauseWorld ~= false then
    self.pauseWorld = true
  end
  
  
  
  
  self.timeDisplayed = love.timer.getTime( )
  
  if not file_exists("dialogue/"..filename..".book") then
    error("Book file '"..filename.."' does not exist!", 2)
  end
  
  
  local lines = love.filesystem.lines("dialogue/"..filename..".book")

  
  
  local i = 1
  
  local k = 1
  for line in lines do 
    local remainder = nil
    
    local thisCharacter = nil
    if line:sub(1,1) == "[" then
      
      j = line:find("]")
      thisCharacter = line:sub(2, j-1)
      if thisCharacter == "" then
        thisCharacter = nil
      end
      if line:sub(j+1,j+1) ~= " " then
        error("Missed a space on line "..k.."!", 2)
      end
      
      line = line:sub(j + 2)

      
    else
      thisCharacter = self.character[k-1]
    end
    self.character[i] = thisCharacter
    
    
    self.page[i], remainder = self:formatPage(line)
    i = i + 1
    
    
    while remainder ~= nil do
      
      self.character[i] = thisCharacter   
      
      self.page[i], remainder = self:formatPage(remainder)
      i = i + 1
      
    end
    k = k + 1
  end
  self.numberOfPages = i
  
end



function DialogueBox:nextPage(specificPage)
  
  if not specificPage then
    if self.currentPage + 1 < self.numberOfPages then
      self.currentPage = self.currentPage + 1
      self.timeDisplayed = love.timer.getTime( )
    else
      self:closeBook()
    end
  else
    if specificPage > self.numberOfPages then
      error("Page does not exist")
    end
    self.currentPage = specificPage
    self.timeDisplayed = love.timer.getTime( )
  end
    
end


function DialogueBox:closeBook()
  self.character = {}
  self.page = {}
  self.currentPage = 1
  self.pauseWorld = false
  table.remove(self.bookQueue, 1)
  self.timeDisplayed = false
  if #self.bookQueue > 0 then
    self:loadBook(self.bookQueue[1])
  else
    camera.targetPoint = nil
    camera.targetZoom = nil
  end
end


function DialogueBox:formatPage(page)
  
  
  local wordTable = {}
  for word in page:gmatch("%S+") do 
    table.insert(wordTable, word)
  end
  
  local i = 1
  
  local pageLines = {}
  
  
  local line = 1
  local wordIndex = 1
  local lineLength = 0
  local newPage = true
  pageLines[line] = {}
  
  local maxLines = self.height / (font:getHeight()*self.fontScale)
  
  while line <= maxLines and wordIndex <= #wordTable do
    local word = wordTable[1]
    local wordLength = font:getWidth(word)
    
    if (lineLength + wordLength + font:getWidth(" "))*self.fontScale <= self.width then
      -- If line isn't exceeded
      if newLine then
        lineLength = lineLength + wordLength
      else
        lineLength = lineLength + font:getWidth(" ") + wordLength
      end
      newPage = false
    else
      -- If line is exceeded
      lineLength = wordLength
      line = line + 1
      if line > maxLines then
        break
      end
      pageLines[line] = {}
      
    end
    table.insert(pageLines[line], word)
    table.remove(wordTable, 1)
  end
  
  
  
  --Convert 2D word table to string
  
  local formattedString = ""
  for i, line in ipairs(pageLines) do
    for j, word in ipairs(line) do
      if i == 1 and j == 1 then
        formattedString = word
      elseif j == 1 then
        formattedString = formattedString ..word
      else
        formattedString = formattedString .." ".. word
      end
    end
    formattedString = formattedString .."\n"
  end
  
  local remainder = nil
  if #wordTable > 0 then
    for i, word in ipairs(wordTable) do
      if i == 1 then
        remainder = word
      else
        remainder = remainder.." "..word
      end
    end
  end
  


  return formattedString, remainder
  
  
end


function DialogueBox.wordTableToString(table, i)
  local returnString = table[1]
  local remainder = nil

  for word = 2, i do
    returnString = returnString.." "..table[word]
  end
  
  
  if i ~= #table then
    remainder = table[i+1]
    
    for word = i+2, #table do
      remainder = remainder.." "..table[word]
    end
  end
  
  return returnString, remainder
end

function string.specialSubstring(inputString, numberOfCharacters)
  
  local ignoredCharacters = {}
  local numberOfIgnoredCharacters = 0
  ignoredCharacters[" "] = true
  ignoredCharacters["\n"] = true
  
  for i = 1, #string do
    if numberOfCharacters < i then break end
    
    
    local byteoffset = utf8.offset(inputString, i) or 0
    
    if ignoredCharacters[string.sub(inputString, i - byteoffset, i - byteoffset)] then
      numberOfIgnoredCharacters = numberOfIgnoredCharacters + 1
    end
  end
  
  
  
  local cutPosition = numberOfCharacters + numberOfIgnoredCharacters
  
  
  local byteoffset = (utf8.offset(inputString, cutPosition+1) or 0) - 1
  
  return (string.sub(inputString, 1, byteoffset))
  
  
  
  
  
  
  
  

  --return string.sub(string, 1, cutPosition - byteoffset)
  
  --return string.sub(string, 1, cutPosition)
end






