Level = Object:extend()


function Level:new(name)
  self.name = name
  self.stage = "main"
  self.polygons = {}
  self.darkZones = {}
end


function Level:setStage(stage)

  self.stage = stage
  if stage == "flow" then
    world:setGravity(0, 0)
    player1.body:setLinearDamping(0.1)
    camera.maxZoom = 1
  else
    camera.maxZoom = 2.5
    player1.body:setLinearDamping(0)
    world:setGravity(0, gravity)
  end
end


function Level:deload()

  local deleteTable = copyTable(drawTable)
  for i, v in ipairs(deleteTable) do

    if v.delete then
      v:delete()
    end
    fireflyTable = {}
    
  end
  flushDrawTable()

end


function love.filedropped(file)

  if builderEnabled then
    file:open("r")
    local data = file:read()
    currentLevel:loadLevel(nil, true, data)
  end
end


local function reduceObject(object, attributesCpy)
  local reducedObject = {}
  reducedObject.x = object.x
  reducedObject.y = object.y
  reducedObject.attributes = attributesCpy
  reducedObject.activeStage = object.activeStage
  
  return reducedObject
end


function Level:saveLevel(filename)

  local gatheredObjects = {}
  gatheredObjects.rawPolygons = {}
  gatheredObjects.rawProps = {}
  gatheredObjects.rawLogicProps = {} --YEEYAYAAAAHAHHHHHHHH
  gatheredObjects.rawLightEmitters = {}
  gatheredObjects.rawTalkatives = {}
  gatheredObjects.rawDoors = {}
  gatheredObjects.rawBraziers = {}  
  gatheredObjects.rawGrass = {}
  gatheredObjects.fireflies = fireflyTable
  gatheredObjects.rawRainEmitters = {}
  gatheredObjects.rawDirectionStones = {}
  gatheredObjects.rawDarkZones = {}
  gatheredObjects.rawSqueezes = {}

  for i, v in ipairs(drawTable) do

    local attributesCpy
    if v.attributes then
      attributesCpy = copyTable(v.attributes)
      --Don't save names which are blank
      if attributesCpy.name and attributesCpy.name == "" then
        attributesCpy.name = nil
      end
    end



    if v.type == "levelPolygon" then

      local reducedPolygon = reduceObject(v, attributesCpy)
      reducedPolygon.points = v.points
      reducedPolygon.originalX = v.originalX
      reducedPolygon.originalY = v.originalY

      --TODO - Make sure undrawn polygons don't have meshes 
      if v.mesh then
        reducedPolygon.pixpolPoints = {}
        for i = 1, v.mesh:getVertexCount(), 1 do
          local x, y = v.mesh:getVertex(i)
          table.insert(reducedPolygon.pixpolPoints, x)
          table.insert(reducedPolygon.pixpolPoints, y)
        end
      end
      table.insert(gatheredObjects.rawPolygons, reducedPolygon)

    elseif v.type == "darkZone" then

      local reducedDarkZone = reduceObject(v, attributesCpy)
      reducedDarkZone.points = v.points

      table.insert(gatheredObjects.rawDarkZones, reducedDarkZone)
      
    elseif v.type == "prop" then
    
      table.insert(gatheredObjects.rawProps, reduceObject(v, attributesCpy))

    elseif v.type == "logicProp" then

      table.insert(gatheredObjects.rawLogicProps, reduceObject(v, attributesCpy))

    elseif v.type == "talkative" then

      table.insert(gatheredObjects.rawTalkatives, reduceObject(v, attributesCpy))
      
    elseif v.type == "lightEmitter" then
      
      table.insert(gatheredObjects.rawLightEmitters, reduceObject(v, attributesCpy))

    elseif v.type == "door" then

      table.insert(gatheredObjects.rawDoors, reduceObject(v, attributesCpy))
      
    elseif v.type == "slidingDoor" then

      table.insert(gatheredObjects.rawDoors, reduceObject(v, attributesCpy))

    elseif v.type == "brazier" then

      table.insert(gatheredObjects.rawBraziers, reduceObject(v, attributesCpy))

    elseif v.type == "rainEmitter" then

      table.insert(gatheredObjects.rawRainEmitters, reduceObject(v, attributesCpy))

    elseif v.type == "directionStone" then
    
      table.insert(gatheredObjects.rawDirectionStones, reduceObject(v, attributesCpy))

    elseif v.type == "grass" then

      local reducedGrass = {}
      reducedGrass.points = v.points
      reducedGrass.flowerCoverage = v.flowerCoverage
      reducedGrass.seed = v.seed
      reducedGrass.activeStage = v.activeStage
      table.insert(gatheredObjects.rawGrass, reducedGrass)
    
    elseif v.type == "squeeze" then
    
      local reducedSqueeze = {}
      reducedSqueeze.points = v.points
      reducedSqueeze.attributes = attributesCpy
      table.insert(gatheredObjects.rawSqueezes, reducedSqueeze)
    
    end
  end
  

  serialized = lume.serialize(gatheredObjects)


  --Save file to appdata
  love.filesystem.createDirectory("levels")
  love.filesystem.createDirectory("levels/autosaves")

  local success, message = love.filesystem.write("levels/"..filename..".cq", serialized)

  if success then
    return 1
  else
    cout(message)
    return 0
  end
end


function Level:loadLevel(filename, deload, rawFile)

  deload = deload or true  
  if deload then
    self:deload()
  end


  local fileData = rawFile or love.filesystem.read("levels/"..filename..".cq")

  if not fileData then
    error("Error - filedata could not be read", 2)
  end


  local loadedObjects = lume.deserialize(fileData)

  for tableName, entityTable in pairs(loadedObjects) do

    if tableName == "fireflies" then
      for i, v in ipairs(entityTable) do
        local result = Firefly(v.x, v.y)
        result.activeStage = v.activeStage
      end

    elseif tableName == "rawPolygons" then
      for i, polygon in ipairs(entityTable) do
        local thisPoly = LevelPolygon(polygon.x, polygon.y, polygon.points, polygon.attributes, polygon.pixpolPoints)

        if polygon.originalX then
          thisPoly.originalX = polygon.originalX
          thisPoly.originalY = polygon.originalY
        end

        thisPoly.activeStage = polygon.activeStage or "main"

        thisPoly:setAttributes()
      end
    elseif tableName == "rawDarkZones" then
      for i, darkZone in ipairs(entityTable) do
        local thisDarkZone = DarkZone(darkZone.x, darkZone.y, darkZone.points, darkZone.attributes)


        thisDarkZone.activeStage = darkZone.activeStage or "main"

        thisDarkZone:setAttributes()
      end

    elseif tableName == "rawGrass" then
      for i, grass in ipairs(entityTable) do
        local newGrass = Grass()
        newGrass.points         = grass.points
        newGrass.flowerCoverage = grass.flowerCoverage
        newGrass.seed           = grass.seed
        newGrass.finalised      = true
        newGrass.activeStage = grass.activeStage or newGrass.activeStage
      end
    elseif tableName == "rawSqueezes" then
      for i, squeeze in ipairs(entityTable) do
        local newSqueeze = Squeeze(nil, nil, squeeze.attributes)
        
        newSqueeze.points = {}
        for i, v in ipairs(squeeze.points) do
          table.insert(newSqueeze.points, Vec2(v.x, v.y))
        end
        
        newSqueeze.activeStage = squeeze.activeStage or newSqueeze.activeStage
      end
    else --If not any of these, assume it's a prop

      local functionName = string.sub(tableName, 4, -2)

      for i, prop in ipairs(entityTable) do
        
        local result = _G[functionName](prop.x, prop.y, prop.attributes)
    
        result.activeStage = prop.activeStage or result.activeStage
        
        if type(result.activeStage) ~= "string" then
          result.activeStage = "main"
        end
      end
    end
  end

end





