#define MAX_LIGHTS 64

extern Image normalMap;
extern Image depthMap;
extern Image ambientMap;

extern vec3 lights[MAX_LIGHTS]; 
extern float lightIntensities[MAX_LIGHTS];
extern vec3 lightColours[MAX_LIGHTS];
extern bool flipX;
extern bool flipY;
extern float cameraZoom;
extern float rotation;

vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
{
  vec4 pixel = Texel(texture, texture_coords);
  
  
  vec3 pixel3 = vec3(pixel.r, pixel.g, pixel.b);
  
  if (pixel3.r == 0.0 && pixel3.g == 0.0 && pixel3.b == 0.0)
  {
    discard;
    return pixel;
  }
  
  //Since I don't want to use a different pixel format but want brightness higher than 1, I'll divide the intensity in set ambient and multiply it here.
  vec3 ambientLight = Texel(ambientMap, texture_coords).rgb * 4.0;
  
  float screenZ = Texel(depthMap, texture_coords).r;
  
  vec4 normalPixel = Texel(normalMap, texture_coords);
  vec3 normal = normalPixel.rgb;
  normal = (normal-0.5)*2.0;
  
  //Rotate normal
  
  
  float cosR = cos(rotation);
  float sinR = sin(rotation);
  
  normal = vec3(cosR*normal.x - sinR*normal.y, sinR*normal.x + cosR*normal.y, normal.z);
  
  
  
  if (flipX) normal.x = -normal.x;
  if (flipY) normal.y = -normal.y;
  
  //Now iterate through lights
    vec3 pointResult = vec3(0.0);
  
  
  
  for (int i = 0; i < MAX_LIGHTS; i++)
  {
    
    float lightIntensity = lightIntensities[i];
    
    //if (lightIntensity == 0.0) break; // If the light intensity is 0, assume that you reached the end of the array
    
    
    vec3 light          = lights[i];
    vec3 lightColour    = lightColours[i];
    
    light.z =  (screenZ - light.z) * cameraZoom;
    
    
    vec3 distance = light - vec3(screen_coords.x, screen_coords.y, 0.0);
    
    float lightFalloff = 1.0/pow(length(distance)/cameraZoom, 2.0);
    vec3 lightDirection = normalize(distance);
      
    float dotProduct = max((dot(lightDirection, normal))+0.1, 0.0);
    
    pointResult += dotProduct*pixel3*lightColour*lightFalloff*lightIntensity*2000.0;
    
  }
  
  //Ambient light
  
  vec3 ambientResult = pixel3 * ambientLight;
  
  float alpha = pixel.a * color.a;
  
  vec4 finalResult = vec4((ambientResult + pointResult)/alpha, alpha);
 
  
  return finalResult;
}