StartMenuButton = Object:extend()


local startMenu = require "startMenu"

function StartMenuButton:new(x, y, scale, selected)
  self.x = x
  self.y = y
  self.scale = scale or 1
  self.selected = false
  self.filledTriangles = 0
  table.insert(startMenu.buttons, self)
  self.active = true
  self.closeAnimationTime = 0
  self.closing = false
end


function StartMenuButton:update(dt)

  if self.closing then
    self.closeAnimationTime = self.closeAnimationTime + dt
  end

end


function StartMenuButton:click(parentManu)
  if self.selected and self.active then
    self:onClick(parentManu)
  end

  return self.selected
end


function StartMenuButton:onClick(parentManu)

  local positiveSound = sound.effect.ui.positive:clone()
  positiveSound:play()
  self:closeAnimation()
  self.active = false
  parentManu:fadeOut()
end


function StartMenuButton:closeAnimation()
  self.closeAnimationTime = 0
  self.closing = true

end

function StartMenuButton:getScale()
  return self.scale * (window.height/1080)
end


function StartMenuButton:getRadius(i)
  i = i or 0
  return math.abs((100 - 10*i) - self.closeAnimationTime*100)*self:getScale() 
end


function StartMenuButton:getPosition()

  return self.x * (window.width/1920), self.y * (window.height/1080)

end


function StartMenuButton:draw()

  local widthScale  = window.width/1920
  local heightScale = window.height/1080
  local x, y  = self:getPosition()
  local scale = self:getScale()

  local r = math.sin(love.timer.getTime()*0.25)*6
  local t = love.timer.getTime()*0.15


  local filledTriangles = 10-- self.filledTriangles



  for i = 0, 9 do

    local lineThickness = (4 - 0.2*i)*scale

    local radius = self:getRadius(i)

    local triangleR = r * (-0.2 + 0.05*i) + t + self.closeAnimationTime

    local transparency = 1 - i/9 - (self.closeAnimationTime*0.3)^2

    love.graphics.setColor(1,1,1, transparency)
    love.graphics.triangle("line", x, y, triangleR, radius - lineThickness*2, lineThickness)

    if self.selected then
      love.graphics.triangle("fill", x, y, triangleR, radius, lineThickness)
    end
  end

  local label = "Start"

  love.graphics.setColor(29/255, 57/255, 112/255, 1 - self.closeAnimationTime*1.5)

  local labelLength = 150 * scale
  local printScale = labelLength/font:getWidth(label) + self.closeAnimationTime*1.5

  local xOffset = font:getWidth(label)/2
  local yOffset = font:getHeight()/2
  love.graphics.print(label, x, y, r*0.02, printScale, printScale, xOffset, yOffset)


end